Package: ggspavis
Version: 1.19.0
Title: Visualization functions for spatial transcriptomics data
Description: Visualization functions for spatial transcriptomics data. Includes 
    functions to generate several types of plots, including spot plots, feature 
    (molecule) plots, reduced dimension plots, spot-level quality control (QC) 
    plots, and feature-level QC plots, for datasets from the 10x Genomics 
    Visium and other technological platforms. Datasets are assumed to be in 
    either SpatialExperiment or SingleCellExperiment format.
Authors@R: c(
    person("Lukas M.", "Weber", 
        email = "lmweb012@gmail.com", 
        role = c("aut", "cre"), 
        comment = c(ORCID = "0000-0002-3282-1730")), 
    person("Helena L.", "Crowell", 
        email = "helena.crowell@uzh.ch", 
        role = c("aut"), 
        comment = c(ORCID = "0000-0002-4801-1767")), 
    person("Yixing E.", "Dong", 
        email = "estelladong729@gmail.com", 
        role = c("aut"), 
        comment = c(ORCID = "0009-0003-5115-5686")))
URL: https://github.com/lmweber/ggspavis
BugReports: https://github.com/lmweber/ggspavis/issues
License: MIT + file LICENSE
Encoding: UTF-8
biocViews: Spatial, SingleCell, Transcriptomics, GeneExpression,
        QualityControl, DimensionReduction
Depends: ggplot2
Imports: SpatialExperiment, SingleCellExperiment, SummarizedExperiment,
        ggside, grid, ggrepel, RColorBrewer, scales, grDevices,
        methods, stats
VignetteBuilder: knitr
Suggests: BiocStyle, rmarkdown, knitr, OSTA.data, VisiumIO, arrow,
        STexampleData, BumpyMatrix, scater, scran, uwot, testthat,
        patchwork
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/ggspavis
git_branch: devel
git_last_commit: 7872157
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:55:27 UTC; biocbuild
Author: Lukas M. Weber [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3282-1730>),
  Helena L. Crowell [aut] (ORCID:
    <https://orcid.org/0000-0002-4801-1767>),
  Yixing E. Dong [aut] (ORCID: <https://orcid.org/0009-0003-5115-5686>)
Maintainer: Lukas M. Weber <lmweb012@gmail.com>

# Generated by roxygen2: do not edit by hand

export(plotCoords)
export(plotDimRed)
export(plotFeatureQC)
export(plotMolecules)
export(plotObsQC)
export(plotQC)
export(plotSpotQC)
export(plotSpots)
export(plotVisium)
importFrom(RColorBrewer,brewer.pal)
importFrom(SingleCellExperiment,'reducedDim<-')
importFrom(SingleCellExperiment,counts)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SpatialExperiment,'imgData<-')
importFrom(SpatialExperiment,imgData)
importFrom(SpatialExperiment,imgRaster)
importFrom(SpatialExperiment,scaleFactors)
importFrom(SpatialExperiment,spatialCoords)
importFrom(SpatialExperiment,spatialCoordsNames)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_gradient)
importFrom(ggplot2,scale_color_gradientn)
importFrom(ggplot2,scale_color_identity)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_color_viridis_c)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_fill_identity)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_fill_viridis_c)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_reverse)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggrepel,geom_text_repel)
importFrom(ggside,geom_xsidehistogram)
importFrom(ggside,geom_ysidehistogram)
importFrom(grDevices,colorRampPalette)
importFrom(grid,rasterGrob)
importFrom(methods,as)
importFrom(methods,is)
importFrom(scales,hue_pal)
importFrom(stats,median)
importFrom(stats,setNames)

# ggspavis

[![R build status](https://github.com/lmweber/ggspavis/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/lmweber/ggspavis/actions)

<img src="inst/extdata/ggspavis.png" width="200" align="right"/>

The `ggspavis` package contains visualization functions for spatial transcriptomics data, including functions to generate several types of plots, including spot plots, feature (molecule) plots, reduced dimension plots, spot-level quality control (QC) plots, and feature-level QC plots, for datasets from the 10x Genomics Visium and other technological platforms. Datasets are assumed to be in either [SpatialExperiment](https://bioconductor.org/packages/SpatialExperiment) or [SingleCellExperiment](https://bioconductor.org/packages/SingleCellExperiment) Bioconductor format.

The package is available from [Bioconductor](https://bioconductor.org/packages/ggspavis).


## Installation

The release version of the package can be installed from Bioconductor:

```
install.packages("BiocManager")
BiocManager::install("ggspavis")
```

The latest development version can be installed either from the development version of Bioconductor or from GitHub:

```
remotes::install_github("lmweber/ggspavis")
```


## Tutorial

A vignette containing a tutorial and examples is available from the [Bioconductor](https://bioconductor.org/packages/ggspavis) package page.


## Citation

- Righell D.\*, Weber L.M.\*, Crowell H.L.\*, Pardo B., Collado-Torres L., Ghazanfar S., Lun A.T.L., Hicks S.C.\*, and Risso D.\* (2022). *SpatialExperiment: infrastructure for spatially resolved transcriptomics data in R using Bioconductor.* [Bioinformatics](https://academic.oup.com/bioinformatics/advance-article/doi/10.1093/bioinformatics/btac299/6575443), btac299.

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = TRUE)

## ----message=FALSE, warning=FALSE---------------------------------------------
library(ggspavis)
library(STexampleData)
library(patchwork)
library(scater)
library(scran)
library(OSTA.data)
library(VisiumIO)
library(SpatialExperiment)

## ----loadslide, message=FALSE, warning=FALSE----------------------------------
spe_slide <- STexampleData::SlideSeqV2_mouseHPC()
spe_slide$loglibsize <- log1p(colSums(counts(spe_slide)))

## ----pltcoordsslide, fig.width=10, fig.height=3, warning=FALSE, message=FALSE----
(plotCoords(spe_slide, annotate = "celltype", 
            in_tissue = NULL, point_size = 0.1) + 
  guides(color=guide_legend(override.aes = list(size = 3), ncol = 2)) |
  plotCoords(spe_slide, annotate = "loglibsize", 
             in_tissue = NULL, point_size = 0.1) + 
  scale_color_gradient(name = "Log library size") + ggtitle("")) +
  plot_annotation(title = 'Slide-seq V2 Mouse Brain')

## ----loadvismb, message=FALSE-------------------------------------------------
# load data in SpatialExperiment format
spe_vm <- Visium_mouseCoronal()
rownames(spe_vm) <- rowData(spe_vm)$gene_name
colData(spe_vm)$sum <- colSums(counts(spe_vm))

## ----vismbvislibsize, message=FALSE, warning=FALSE, fig.width=8, fig.height=3.5----
p1 <- plotVisium(spe_vm, annotate = "sum", highlight = "in_tissue", 
                 legend_position = "none")
p2 <- plotVisium(spe_vm, annotate = "sum", highlight = "in_tissue", 
                 pal = "darkred") + 
  guides(fill = guide_colorbar(title = "Libsize"))

# display panels using patchwork
p1 | p2

## ----vismbgevisge, warning=FALSE, fig.width=8, fig.height=3.5-----------------
p1 <- plotVisium(spe_vm, annotate = "Gapdh", highlight = "in_tissue")
p2 <- plotVisium(spe_vm, annotate = "Mbp", highlight = "in_tissue")

# display panels using patchwork
p1 | p2

## ----vismbonlyspots, warning=FALSE, fig.width=8, fig.height=3.5---------------
p1 <- plotVisium(spe_vm, annotate = "Mbp", 
                 highlight = "in_tissue", image = FALSE)
p2 <- plotVisium(spe_vm, annotate = "Mbp", 
                 highlight = "in_tissue", spots = FALSE)

# display panels using patchwork
p1 | p2

## ----vismbcoordsge, fig.width=6, fig.height=3---------------------------------
p1 <- plotCoords(spe_vm, annotate = "Gapdh")
p2 <- plotCoords(spe_vm, annotate = "Mbp", pal = "viridis")

# display panels using patchwork
p1 | p2

## ----loaddlpfc, message=FALSE-------------------------------------------------
# load data in SpatialExperiment format
spe <- Visium_humanDLPFC()
rownames(spe) <- rowData(spe)$gene_name
colData(spe)$libsize <- colSums(counts(spe))

## ----dlpfcgroundtruth, message=FALSE, out.width="60%"-------------------------
plotVisium(spe, annotate = "ground_truth", highlight = "in_tissue", 
           pal = "libd_layer_colors")

## ----testpalettes, fig.width=6, fig.height=3----------------------------------
p1 <- plotCoords(spe, annotate = "ground_truth", pal = "Okabe-Ito") +
  ggtitle("Reference")
p2 <- plotCoords(spe, annotate = "libsize", pal = "rainbow") + 
  ggtitle("Library size")

# display panels using patchwork
p1 | p2

## ----qcmetrics, message=FALSE-------------------------------------------------
# calculate QC metrics using scater
spe <- addPerCellQCMetrics(spe, 
  subsets = list(mito = grepl("(^MT-)|(^mt-)", rowData(spe)$gene_name)))

# apply QC thresholds
colData(spe)$low_libsize <- colData(spe)$sum < 400 | colData(spe)$detected < 400
colData(spe)$high_mito <- colData(spe)$subsets_mito_percent > 30

## ----obsqclowlibsize, fig.width=8, fig.height=2.5-----------------------------
p1 <- plotObsQC(spe, plot_type = "histogram", 
                x_metric = "sum", annotate = "low_libsize")
p2 <- plotObsQC(spe, plot_type = "violin", 
                x_metric = "sum", annotate = "low_libsize", point_size = 0.1)
p3 <- plotObsQC(spe, plot_type = "spot", in_tissue = "in_tissue", 
                annotate = "low_libsize", point_size = 0.2)

# display panels using patchwork
p1 | p2 | p3

## ----obsqchighmt, fig.width=8, fig.height=2.5---------------------------------
p1 <- plotObsQC(spe, plot_type = "histogram", 
                x_metric = "subsets_mito_percent", annotate = "high_mito")
p2 <- plotObsQC(spe, plot_type = "violin", 
                x_metric = "subsets_mito_percent", annotate = "high_mito", 
                point_size = 0.1)
p3 <- plotObsQC(spe, plot_type = "spot", in_tissue = "in_tissue", 
                annotate = "high_mito", point_size = 0.2)

# display panels using patchwork
p1 | p2 | p3

## ----obsqcscatter, out.width="60%", warning=FALSE, message=FALSE--------------
plotObsQC(spe, plot_type = "scatter", 
          x_metric = "subsets_mito_percent", y_metric = "sum", 
          x_threshold = 30, y_threshold = 400)

## ----featqc, warning=FALSE, message=FALSE, fig.width=8, fig.height=3----------
rowData(spe)$feature_sum <- rowSums(counts(spe))
rowData(spe)$low_abundance <- rowSums(counts(spe) > 0) < 20

p1 <- plotFeatureQC(spe, plot_type = "histogram", 
                    x_metric = "feature_sum", annotate = "low_abundance")
p2 <- plotFeatureQC(spe, plot_type = "violin", 
                    x_metric = "feature_sum", annotate = "low_abundance")

# display panels using patchwork
p1 | p2

## ----loadvhd8sub, message=FALSE-----------------------------------------------
# retrieve dataset from OSF repo
id <- "VisiumHD_HumanColon_Oliveira"
pa <- OSTA.data_load(id)
dir.create(td <- tempfile())
unzip(pa, exdir=td)

# read 8um bins into 'SpatialExperiment'
vhd8 <- TENxVisiumHD(spacerangerOut=td, processing="filtered", format="h5", 
                     images="lowres", bin_size="008") |> import()
# subset
vhd8 <- vhd8[, spatialCoords(vhd8)[, 1] * scaleFactors(vhd8) > 430 & 
               spatialCoords(vhd8)[, 1] * scaleFactors(vhd8) < 435 &
               spatialCoords(vhd8)[, 2] * scaleFactors(vhd8) > 127 & 
               spatialCoords(vhd8)[, 2] * scaleFactors(vhd8) < 132]
rownames(vhd8) <- rowData(vhd8)$Symbol
vhd8

## ----vhd8coord, warning=FALSE, fig.width=5, fig.height=2.5--------------------
plotCoords(vhd8, point_shape=15, point_size = 1.7, annotate="PIGR") | 
  plotVisium(vhd8, point_shape=22, point_size = 2, annotate="PIGR", 
           zoom = TRUE)

## ----imgdataload, message=FALSE, warning=FALSE--------------------------------
spe_xen <- STexampleData::Janesick_breastCancer_Xenium_rep1()
spe_cos <- STexampleData::CosMx_lungCancer()
spe_mer <- STexampleData::MERSCOPE_ovarianCancer()
spe_sta <- STexampleData::STARmapPLUS_mouseBrain()

## ----imgspatial, message=FALSE, fig.width=12, fig.height=3--------------------
plotCoords(spe_xen, annotate = "cell_area", in_tissue = NULL, pal = "magma",
           point_size = 0.15) + ggtitle("Xenium Breast Cancer") |
  plotCoords(spe_cos, annotate = "Area", in_tissue = NULL, pal = "plasma", 
             point_size = 0.1) + ggtitle("CosMx Breast Cancer") |
  plotCoords(spe_mer, annotate = "volume", in_tissue = NULL, 
             pal = c("navyblue", "yellow"),
             point_size = 0.05) + ggtitle("MERSCOPE Ovarian Cancer")

## ----spatialtextby, message=FALSE, fig.width=6, fig.height=3.5----------------
plotCoords(spe_sta, annotate = "Main_molecular_tissue_region", in_tissue = NULL,
           point_size = 0.1, text_by = "Main_molecular_tissue_region",
           text_by_size = 4, text_by_color = "#2d2d2d")

## ----getumap------------------------------------------------------------------
spe_sta <- logNormCounts(spe_sta)
dec <- modelGeneVar(spe_sta)
hvg <- getTopHVGs(dec, n=3e3)
spe_sta <- runPCA(spe_sta, subset_row=hvg)
spe_sta <- runUMAP(spe_sta, dimred="PCA")

## ----pltdimred, message=FALSE, fig.width=6, fig.height=3.5--------------------
plotDimRed(spe_sta, plot_type = "UMAP",
           annotate = "Main_molecular_tissue_region", 
           text_by = "Main_molecular_tissue_region",
           text_by_size = 3, text_by_color = "#2d2d2d")

## -----------------------------------------------------------------------------
sessionInfo()

#' plotObsQC
#' 
#' Plotting functions for spatial transcriptomics data.
#' 
#' Function to create observation-level quality control (QC) plots for spatial
#' transcriptomics data.
#' 
#' The following types of observation-level (i.e. spot-level or cell-level) QC
#' plots are available:
#' 
#' \itemize{
#' \item Histogram (\code{plot_type = "histogram"}) for a single QC metric, e.g.
#' number of UMI counts per spot. For number of counts per spot, the histogram
#' can optionally highlight selected spots, e.g. spots with low library size.
#' \item Scatter plot (\code{plot_type = "scatter"}) comparing two QC metrics,
#' e.g. number of detected features vs. number of cells per spot, with optional
#' horizontal and vertical lines highlighting QC filtering thresholds.
#' \item Spot plot (\code{plot_type = "spot"}) showing spots in spatial x-y
#' coordinates, e.g. highlighting selected spots that do not meet filtering
#' thresholds.
#' \item Violin plot (\code{plot_type = "violin"}) for a single QC metric, e.g.
#' number of UMI counts per spot. For number of counts per spot, the violin plot
#' can optionally highlight selected spots, e.g. spots with low library size.
#' }
#' 
#' For feature-level (i.e. gene-level) QC plots, see
#' \code{\link{plotFeatureQC}}.
#' 
#' 
#' @param spe Input data, assumed to be a \code{SpatialExperiment} or
#'   \code{SingleCellExperiment} object.
#' 
#' @param plot_type Type of QC plot. Options are "histogram", "scatter", "spot",
#'   and "violin". See Details for additional details.
#' 
#' @param x_coord Name of column in \code{spatialCoords} (for a
#'   \code{SpatialExperiment} input object) or \code{colData} (for a
#'   \code{SingleCellExperiment} input object) containing x coordinates. Default
#'   = NULL (for a \code{SpatialExperiment}, the first column of
#'   \code{spatialCoords} will be selected in this case). Used for spot plots.
#' 
#' @param y_coord Name of column in \code{spatialCoords} (for a
#'   \code{SpatialExperiment} input object) or \code{colData} (for a
#'   \code{SingleCellExperiment} input object) containing y coordinates. Default
#'   = NULL (for a \code{SpatialExperiment}, the second column of
#'   \code{spatialCoords} will be selected in this case). Used for spot plots.
#' 
#' @param x_metric Name of column in \code{colData} containing QC metric to plot
#'   on x-axis. Required for histograms, scatter plots, and violin plots.
#' 
#' @param y_metric Name of column in \code{colData} containing QC metric to plot
#'   on y-axis. Required for histograms, scatter plots, and violin plots.
#' 
#' @param x_threshold QC filtering threshold on x-axis metric to highlight with
#'   vertical line. Default = NULL. Optional argument used for scatter plots.
#' 
#' @param y_threshold QC filtering threshold on y-axis metric to highlight with
#'   horizontal line. Default = NULL. Optional argument used for scatter plots.
#' 
#' @param trend Whether to show smoothed trend line (loess). Default = TRUE.
#'   Optional argument used for scatter plots.
#' 
#' @param marginal Whether to show marginal histograms. Default = TRUE. Optional
#'   argument used for scatter plots.
#' 
#' @param annotate Name of column in \code{colData} identifying selected spots
#'   that do not meet QC filtering thresholds, which will be highlighted on a
#'   histogram, spot plot, or violin plot. Default = NULL. Optional argument
#'   used for histograms, spot plots, and violin plots.
#' 
#' @param in_tissue Name of column in \code{colData} identifying spots over
#'   tissue (e.g. "in_tissue" for 10x Genomics Visium datasets). If this
#'   argument is provided, only spots over tissue will be shown. Default = NULL.
#'   Optional argument used for spot plots.
#' 
#' @param legend_point_size Legend point size. Default = 3. Optional argument
#'   used for spot plots.
#' 
#' @param n_bins Number of bins for histograms. Default = 100. Optional argument
#'   used for histograms.
#' 
#' @param point_size Point size. Default = 0.3. Optional argument for scatter
#'   plots, spot plots, and violin plots. Suggested values: 0.5 for scatter
#'   plots, 0.3 for spot plots, 0.1 for violin plots.
#' 
#' @param y_reverse Whether to reverse y coordinates. This is usually required
#'   for 10x Genomics Visium datasets when using the default coordinate values.
#'   Default = TRUE. Set to FALSE if not needed, e.g. for other platforms.
#'   Optional argument used for spot plots.
#' 
#' 
#' @return Returns a ggplot object, which may be further modified using ggplot
#'   functions.
#' 
#' 
#' @importFrom SpatialExperiment spatialCoords
#' @importFrom SummarizedExperiment rowData colData
#' @importFrom ggplot2 ggplot geom_histogram geom_point geom_vline geom_hline
#'   geom_smooth geom_violin geom_jitter scale_fill_manual scale_color_manual
#'   xlab labs coord_fixed theme_bw theme element_text element_blank guides
#'   scale_y_reverse
#' @importFrom ggside geom_xsidehistogram geom_ysidehistogram
#' 
#' 
#' @export
#' 
#' @author Lukas M. Weber and Yixing E. Dong
#' 
#' @examples
#' library(STexampleData)
#' spe <- Visium_humanDLPFC()
#' 
#' colData(spe)$sum <- colSums(counts(spe))
#' colData(spe)$low_libsize <- colData(spe)$sum < 400
#' 
#' plotObsQC(spe, plot_type = "histogram", x_metric = "sum", annotate = "low_libsize")
#' plotObsQC(spe, plot_type = "scatter", x_metric = "sum", y_metric = "cell_count")
#' plotObsQC(spe, plot_type = "spot", annotate = "low_libsize", in_tissue = "in_tissue")
#' plotObsQC(spe, plot_type = "violin", x_metric = "sum", annotate = "low_libsize")
#' 
plotObsQC <- function(spe, 
                      plot_type = c("histogram", "scatter", "spot", "violin"), 
                      x_coord = NULL, y_coord = NULL, 
                      x_metric = NULL, y_metric = NULL, 
                      x_threshold = NULL, y_threshold = NULL, 
                      trend = TRUE, marginal = TRUE, 
                      annotate = NULL, in_tissue = NULL, 
                      legend_point_size = 3, 
                      n_bins = 100, point_size = 0.3, 
                      y_reverse = TRUE) {
  
  # check validity of arguments
  plot_type <- match.arg(plot_type)
  
  if (!is.null(in_tissue)) {
    stopifnot(is.character(in_tissue))
  }
  
  if (!is.null(annotate)) {
    stopifnot(is.character(annotate))
    if (!(annotate %in% colnames(colData(spe)))) {
      stop("'annotate' should be the name of a column in colData")
    }
  }
  
  # set up data frame for plotting
  if (is(spe, "SpatialExperiment")) {
    # select default columns of x and y coordinates
    if (is.null(x_coord)) x_coord <- colnames(spatialCoords(spe))[1]
    if (is.null(y_coord)) y_coord <- colnames(spatialCoords(spe))[2]
    df <- cbind(data.frame(colData(spe), check.names = FALSE), 
                data.frame(spatialCoords(spe), check.names = FALSE))
  } else if (is(spe, "SingleCellExperiment")) {
    if (is.null(x_coord) || is.null(y_coord)) {
      stop("Please provide 'x_coord' and 'y_coord' arguments to specify ", 
           "columns in colData containing x and y coordinates.")
    }
    df <- data.frame(colData(spe), check.names = FALSE)
  }
  
  # for histogram, spot, or violin plots
  if (!is.null(annotate)) {
    if (!is.logical(df[[annotate]])) {
      stop("'annotate' should be a logical vector, and 'x_metric' and/or ", 
           "'y_metric' should be vectors of continuous values.")
    }
  }
  
  if (!is.null(in_tissue)) {
    df <- df[df[, in_tissue] == 1, ]
  }
  
  
  # histogram: requires 'x_metric' (continuous), optionally 'annotate' (logical)
  
  if (plot_type == "histogram") {
    
    stopifnot(is.numeric(n_bins))
    
    # histogram showing 'x_metric', optionally colored by 'annotate'
    if (!is.null(annotate)) {
      p <- ggplot(df, aes(x = get(x_metric), fill = get(annotate))) + 
        geom_histogram(bins = n_bins, color = "#e9ecef", alpha = 0.6, 
                       position = "identity") + 
        scale_fill_manual(values = c("gray70", "red")) + 
        labs(fill = annotate)
    } else if (is.null(annotate)) {
      p <- ggplot(df, aes(x = get(x_metric))) + 
        geom_histogram(bins = n_bins, color = "#e9ecef", alpha = 0.6, 
                       position = "identity") + 
        scale_fill_manual(values = c("gray70"))
    }
    
    p <- p + xlab(x_metric) + theme_bw()
  }
  
  
  # scatter: requires 'x_metric' (continuous) and 'y_metric' (continuous),
  # additional optional arguments
  
  if (plot_type == "scatter") {
    
    p <- ggplot(df, aes(x = get(x_metric), y = get(y_metric))) + 
      geom_point(size = point_size) + 
      labs(x = x_coord, 
           y = y_coord) + 
      theme_bw()
    
    if (!is.null(x_threshold)) {
      p <- p + 
        geom_vline(xintercept = x_threshold, color = "red")
    }
    if (!is.null(y_threshold)) {
      p <- p + 
        geom_hline(yintercept = y_threshold, color = "red")
    }
    if (trend) {
      p <- p + 
        geom_smooth(method = "loess", se = FALSE)
    }
    if (marginal) {
      p <- p + 
        geom_xsidehistogram() + 
        geom_ysidehistogram()
    }
  }
  
  
  # spot: requires 'x_coord' (continuous), 'y_coord' (continuous), optionally
  # 'annotate' (logical)
  
  if (plot_type == "spot") {
    
    # spots at 'x_coord' and 'y_coord', optionally colored by 'annotate'
    if (!is.null(annotate)) {
      p <- ggplot(df, aes(x = get(x_coord), y = get(y_coord), color = get(annotate))) + 
        geom_point(size = point_size) + 
        scale_color_manual(values = c("gray85", "red")) + 
        labs(color = annotate) + 
        guides(color = guide_legend(override.aes = list(size = legend_point_size)))
    } else if (is.null(annotate)) {
      p <- ggplot(df, aes(x = get(x_coord), y = get(y_coord))) + 
        geom_point(size = point_size) + 
        scale_color_manual(values = "gray85")
    }
    
    p <- p + 
      coord_fixed() + 
      labs(x = x_coord, 
           y = y_coord) + 
      theme_bw() + 
      theme(panel.grid = element_blank(), 
            axis.title = element_blank(), 
            axis.text = element_blank(), 
            axis.ticks = element_blank())
    
    if (y_reverse) {
      p <- p + scale_y_reverse()
    }
  }
  
  
  # violin: requires 'x_metric' (continuous), optionally 'annotate' (logical)
  
  if (plot_type == "violin") {
    
    df[["dummy"]] <- rep(" ", nrow(df))
    
    p <- ggplot(df, aes(x = get("dummy"), y = get(x_metric), fill = get("dummy"))) + 
      geom_violin(trim = TRUE, alpha = 0.9) + 
      scale_fill_manual(values = c("gray70")) + 
      labs(x = "dummy", 
           y = x_metric, 
           fill = "dummy") + 
      theme_bw() + 
      theme(legend.position = "none", 
            panel.grid = element_blank())
    
    if (is.null(annotate)) {
      # violins for 'x_metric'
      p <- p + 
        geom_jitter(size = point_size)
    } else if (!is.null(annotate)) {
      # violins for 'x_metric', colored by 'annotate' (colors in order FALSE, TRUE)
      p <- p + 
        geom_jitter(aes(color = get(annotate)), size = point_size) + 
        scale_color_manual(values = c("black", "red")) + 
        labs(color = annotate)
    }
  }
  
  # return plot
  p
}


#' @rdname plotObsQC
#' @param ... Not used.
#' @export
plotQC <- function(...) {
  # message when using deprecated function name
  message("The function plotQC() has been replaced with plotObsQC() and ", 
          "plotFeatureQC().")
}


#' @rdname plotObsQC
#' @param ... Not used.
#' @export
plotSpotQC <- function(...) {
  # message when using deprecated function name
  message("The function plotSpotQC() has been replaced with plotObsQC(). Please ", 
          "use the new function name.")
}

#' plotMolecules
#' 
#' Plotting functions for spatial transcriptomics data.
#' 
#' Function to create spot plot for molecule-based datasets, showing spatial
#' locations in x-y coordinates with optional annotations such as expression of
#' a gene.
#' 
#' 
#' @param spe (SpatialExperiment) Input data, assumed to be a
#'   \code{SpatialExperiment} object.
#' 
#' @param molecule Name of mRNA molecule to plot (assumed to match one of the
#'   row names of \code{rowData}).
#' 
#' @param x_coord Name of column in \code{spatialCoords} containing x
#'   coordinates. Default = NULL, which selects the first column of
#'   \code{spatialCoords}.
#' 
#' @param y_coord Name of column in \code{spatialCoords} containing y
#'   coordinates. Default = NULL, which selects the second column of
#'   \code{spatialCoords}.
#' 
#' @param sample_id Name of column in \code{colData} containing sample IDs. This
#'   argument is only required for datasets containing multiple samples (tissue
#'   sections). If provided, samples will be shown in multiple panels using
#'   facetting. Default = NULL.
#' 
#' @param pal Color palette, provided as a vector of length 2 for the low and
#'   high range. Default = c("gray90", "navy").
#' 
#' @param point_size Point size. Default = 0.3.
#' 
#' 
#' @return Returns a ggplot object, which may be further modified using ggplot
#'   functions.
#' 
#' 
#' @importFrom SpatialExperiment spatialCoords
#' @importFrom SingleCellExperiment counts
#' @importFrom ggplot2 ggplot facet_wrap geom_point scale_color_gradient
#'   coord_fixed ggtitle theme_void
#' 
#' @export
#' 
#' @author Lukas M. Weber
#' 
#' @examples
#' library(STexampleData)
#' spe <- seqFISH_mouseEmbryo()
#' plotMolecules(spe, molecule = "Sox2")
#' 
plotMolecules <- function(spe, 
                          molecule =  NULL, 
                          x_coord = NULL, y_coord = NULL, 
                          sample_id = "sample_id", 
                          pal = c("gray90", "navy"), 
                          point_size = 0.3) {
  
  if (!is.null(molecule)) stopifnot(is.character(molecule))
  
  if (is.null(x_coord)) x_coord <- colnames(spatialCoords(spe))[1]
  if (is.null(y_coord)) y_coord <- colnames(spatialCoords(spe))[2]
  
  n_samples <- length(table(colData(spe)[, sample_id]))
  
  mRNA_counts <- as.numeric(counts(spe)[molecule, ])
  stopifnot(length(mRNA_counts) == ncol(spe))
  
  # providing a single value e.g. "navy" will create a vector c("gray95", "navy")
  pal <- .get_pal(pal)
  
  df <- cbind(data.frame(spatialCoords(spe), check.names = FALSE), 
              sum = mRNA_counts)
  
  p <- ggplot(df, aes(x = get(x_coord), y = get(y_coord), color = get("sum"))) + 
    geom_point(size = point_size) + 
    scale_color_gradient(low = pal[1], high = pal[2], trans = "sqrt") + 
    coord_fixed() + 
    labs(color = "sum") + 
    ggtitle(molecule) + 
    theme_void()
  
  if (n_samples > 1) {
    p <- p + facet_wrap(~ sample_id)
  }
  
  p
}

#' plotCoords
#' 
#' Plotting functions for spatial transcriptomics data.
#' 
#' Function to create spot plot showing spatial locations in x-y coordinates
#' with optional annotations such as cluster labels, expression of a gene, or
#' quality control metrics.
#' 
#' 
#' @param spe Input data, assumed to be a \code{SpatialExperiment} or
#'   \code{SingleCellExperiment} object.
#' 
#' @param x_coord Name of column in \code{spatialCoords} (for a
#'   \code{SpatialExperiment} input object) or \code{colData} (for a
#'   \code{SingleCellExperiment} input object) containing x coordinates. Default
#'   = NULL (for a \code{SpatialExperiment}, the first column of
#'   \code{spatialCoords} will be selected in this case).
#' 
#' @param y_coord Name of column in \code{spatialCoords} (for a
#'   \code{SpatialExperiment} input object) or \code{colData} (for a
#'   \code{SingleCellExperiment} input object) containing y coordinates. Default
#'   = NULL (for a \code{SpatialExperiment}, the second column of
#'   \code{spatialCoords} will be selected in this case).
#' 
#' @param sample_id Name of column in \code{colData} containing sample IDs. This
#'   argument is only required for datasets containing multiple samples (tissue
#'   sections). If provided, samples will be shown in multiple panels using
#'   facetting. Default = NULL.
#' 
#' @param in_tissue Name of column in \code{colData} identifying spots over
#'   tissue (e.g. "in_tissue" for 10x Genomics Visium datasets). If this
#'   argument is provided, only spots over tissue will be shown. Default =
#'   "in_tissue". Set to NULL to display all spots.
#' 
#' @param annotate Variable to show as annotations. This may be discrete or
#'   continuous. For a discrete variable (e.g. cluster labels), this should be
#'   the name of a column in \code{colData} containing a character vector or
#'   factor. For a continuous variable (e.g. a gene name), this should be an
#'   entry in \code{feature_names}. Default = NULL.
#' 
#' @param feature_names Name of column in \code{rowData} containing names of
#'   continuous features to plot (e.g. gene names). For example, set to
#'   \code{feature_names = "gene_name"} if gene names are stored in a column
#'   named \code{"gene_name"}. This argument is used if \code{annotate} is a
#'   continuous variable. Default = NULL, in which case the row names of the
#'   input object will be used.
#' 
#' @param assay_name Name of \code{assay} in input object containing values to
#'   plot for a continuous variable. Default = "counts".
#' 
#' @param pal Color palette for annotations. Options for discrete values are
#'   "libd_layer_colors", "Okabe-Ito", or any vector of color names or hex
#'   values. For continuous values, provide a vector of length 2 for the low and
#'   high range, e.g. c("gray90", "navy").
#' 
#' @param point_shape Point shape. Default = 16, which gives a circular shape
#'   suitable for representing, for example, a Visium spot or Xenium cell. A
#'   value of 15 gives a square shape suitable for Visium HD.
#' 
#' @param point_size Point size. Default = 0.3.
#' 
#' @param legend_position Legend position for discrete annotations. Options are
#'   "left", "right", "top", "bottom", and "none". Default = "right".
#' 
#' @param legend_point_size Legend point size for discrete annotations. Default
#'   = 3.
#' 
#' @param show_axes Whether to show axis titles, text, and ticks. Default =
#'   FALSE.
#' 
#' @param y_reverse Whether to reverse y coordinates. This is usually required
#'   for 10x Genomics Visium datasets when using the default coordinate values.
#'   Default = TRUE. Set to FALSE if not needed, e.g. for other platforms.
#' 
#' @param text_by Column name of annotation labels to display over each cluster
#'   of points. This will usually be the same as \code{annotate}. Alternatively,
#'   another column may be used (e.g. with more readable classes or shorter
#'   strings). Only used for discrete \code{annotate}. Default = NULL.
#' 
#' @param text_by_size Text size for annotation labels over each cluster.
#'   Default = 5.
#' 
#' @param text_by_color Color name or hex code for annotation labels. Default =
#'   "black".
#' 
#' 
#' @return Returns a ggplot object, which may be further modified using ggplot
#'   functions.
#' 
#' 
#' @importFrom SpatialExperiment spatialCoords
#' @importFrom SingleCellExperiment reducedDimNames reducedDim
#' @importFrom SummarizedExperiment assay colData rowData
#' @importFrom grDevices colorRampPalette
#' @importFrom RColorBrewer brewer.pal
#' @importFrom scales hue_pal
#' @importFrom stats median
#' @importFrom ggrepel geom_text_repel
#' @importFrom ggplot2 ggplot geom_point facet_wrap coord_fixed theme_bw theme
#'   element_blank scale_color_viridis_c scale_color_gradientn
#'   scale_color_gradient scale_color_manual ggtitle labs guides scale_y_reverse
#'   aes .data
#' 
#' @export
#' 
#' @author Lukas M. Weber and Yixing E. Dong
#' 
#' @examples
#' library(STexampleData)
#' 
#' # discrete annotations
#' spe <- Visium_humanDLPFC()
#' plotCoords(spe, annotate = "ground_truth")
#' 
#' # continuous annotations
#' spe <- Visium_mouseCoronal()
#' plotCoords(spe, annotate = "Gapdh", feature_names = "gene_name")
#' 
plotCoords <- function(spe, x_coord = NULL, y_coord = NULL, 
                       sample_id = NULL, in_tissue = "in_tissue", 
                       annotate = NULL, feature_names = NULL, 
                       assay_name = "counts", 
                       pal = NULL, point_shape = 16, point_size = 0.3, 
                       legend_position = "right", legend_point_size = 3, 
                       show_axes = FALSE, y_reverse = TRUE, 
                       text_by = NULL, text_by_size = 5, 
                       text_by_color = "black") {
  
  # check validity of arguments
  if (!is.null(in_tissue)) {
    stopifnot(is.character(in_tissue))
  }
  
  # get names of continuous features
  if (is.null(feature_names)) {
    feature_nms <- rownames(spe)
  } else {
    stopifnot(ncol(rowData(spe)) > 0)
    stopifnot(feature_names %in% colnames(rowData(spe)))
    feature_nms <- rowData(spe)[, feature_names]
  }
  
  if (!is.null(annotate)) {
    stopifnot(is.character(annotate))
    if (!(annotate %in% c(colnames(colData(spe)), feature_nms))) {
      stop("'annotate' should be either (i) the name of a column in colData ", 
           "or (ii) an entry in either a column named 'feature_names' in ", 
           "rowData or the rownames of the input object")
    }
  }
  
  stopifnot(legend_position %in% c("left", "right", "top", "bottom", "none"))
  
  if (!is.null(sample_id)) {
    stopifnot(sample_id %in% colnames(colData(spe)))
    n_samples <- length(table(colData(spe)[, sample_id]))
  } else {
    n_samples <- NULL
  }
  
  # set up data frame for plotting
  if (is(spe, "SpatialExperiment")) {
    # select default columns of x and y coordinates
    if (is.null(x_coord)) x_coord <- colnames(spatialCoords(spe))[1]
    if (is.null(y_coord)) y_coord <- colnames(spatialCoords(spe))[2]
    df <- cbind(data.frame(colData(spe), check.names = FALSE), 
                data.frame(spatialCoords(spe), check.names = FALSE))
  } else if (is(spe, "SingleCellExperiment")) {
    if (is.null(x_coord) || is.null(y_coord)) {
      stop("Please provide 'x_coord' and 'y_coord' arguments to specify ", 
           "columns in colData containing x and y coordinates.")
    }
    df <- data.frame(colData(spe), check.names = FALSE)
  }
  
  if (!is.null(annotate)) {
    # continuous annotation values
    if (annotate %in% feature_nms) {
      stopifnot(is.character(assay_name))
      ix <- which(feature_nms == annotate)
      df[[annotate]] <- assay(spe, assay_name)[ix, ]
    }
    # discrete annotation values
    if ((annotate %in% colnames(colData(spe))) && 
        (is.character(colData(spe)[, annotate]))) {
      df[[annotate]] <- as.factor(df[[annotate]])
    }
  }
  
  if (!is.null(in_tissue)) {
    df <- df[df[, in_tissue] == 1, ]
  }
  
  # color palettes
  if ((is.null(annotate) || is.numeric(df[[annotate]])) && is.null(pal)) {
    # for continuous values, change NULL to arbitrary color so length(pal) == 1
    pal <- "blue"
  }
  # accepts "libd_layer_colors" and "Okabe-Ito", or arbitrary color palette for NULL
  pal <- .get_pal(pal)
  
  
  # main plot
  
  p <- ggplot(df, aes(x = get(x_coord), y = get(y_coord), 
                      color = if (!is.null(annotate)) get(annotate) else NULL)) + 
    geom_point(size = point_size, shape = point_shape) + 
    labs(x = x_coord, 
         y = y_coord, 
         color = annotate) + 
    coord_fixed() + 
    theme_bw() + 
    theme(legend.position = legend_position, 
          panel.grid = element_blank())
  
  if (!is.null(n_samples) && n_samples > 1) {
    p <- p + facet_wrap(~ sample_id)
  }
  
  
  # additional plot formatting
  
  if (show_axes == FALSE) {
    p <- p + theme(axis.title = element_blank(), 
                   axis.text = element_blank(), 
                   axis.ticks = element_blank())
  }
  
  # color scale
  if (!is.null(annotate)) {
    scaling <- if (is.numeric(df[[annotate]])) {
      # continuous values
      if (length(pal) == 1 && 
          pal %in% c("viridis", "magma", "inferno", "plasma", "cividis", 
                     "rocket", "mako", "turbo")) {
        scale_color_viridis_c(option = pal)
      } else if (length(pal) == 1 && pal == "rainbow") {
        colors <- colorRampPalette(
          colors = rev(x = brewer.pal(n = 11, name = "Spectral")))(100)
        scale_color_gradientn(colors = colors, limits = range(df[[annotate]]))
      } else {
        scale_color_gradient(low = pal[1], high = pal[2])
      }
    } else if (is.factor(df[[annotate]]) || is.character(df[[annotate]])) {
      # discrete values
      if (is.null(pal)) {
        scale_color_manual(name = annotate, 
                           values = hue_pal()(length(unique(df[[annotate]]))))
      } else if (!is.null(pal)) {
        scale_color_manual(values = pal)
      }
    }
  }
  
  # plot title
  if (!is.null(annotate)) {
    if (is.numeric(df[[annotate]])) {
      # continuous values: display plot title but no legend title
      p <- p + 
        scaling + 
        ggtitle(annotate) + 
        labs(color = NULL) + 
        theme(plot.title = element_text(hjust = 0.5))
    } else if (is.factor(df[[annotate]]) || is.character(df[[annotate]])) {
      # discrete values: display legend title but no plot title
      p <- p + 
        scaling + 
        guides(color = guide_legend(override.aes = list(size = legend_point_size)))
    }
  }
  
  # text annotations
  if (!is.null(annotate)) {
    if (is.factor(df[[annotate]]) || is.character(df[[annotate]])) {
      # add text with the median locations of the 'text_by' vector
      if (!is.null(text_by)) {
        by_text_x <- vapply(
          split(df[[x_coord]], df[[text_by]]), 
          median, 
          FUN.VALUE = 0
        )
        by_text_y <- vapply(
          split(df[[y_coord]], df[[text_by]]), 
          median, 
          FUN.VALUE = 0
        )
        p <- p + 
          geom_text_repel(
            data = data.frame(
              x = by_text_x, 
              y = by_text_y, 
              label = names(by_text_x), 
              check.names = FALSE
            ), 
            mapping = aes(x = .data$x, y = .data$y, label = .data$label), 
            size = text_by_size, 
            color = text_by_color
          )
      }
    }
  }
  
  # reverse y axis
  if (y_reverse) {
    p <- p + scale_y_reverse()
  }
  
  # return plot
  p
}


#' @rdname plotCoords
#' @param ... Not used.
#' @export
plotSpots <- function(...) {
  # message when using deprecated function name
  message("The function plotSpots() has been replaced with plotCoords(), which ", 
          "can be used with both sequencing-based and imaging-based data. Please ", 
          "use the new function name.")
}

.get_pal <- function(pal, val) {
  
  if (length(pal) == 1) {
    pal <- switch(pal, 
      "libd_layer_colors" = c(
        "#F0027F", "#377EB8", "#4DAF4A", "#984EA3", 
        "#FFD700", "#FF7F00", "#1A1A1A", "#666666"), 
      "Okabe-Ito" = c(
        "#000000", "#E69F00", "#56B4E9", "#009E73", 
        "#F0E442", "#0072B2", "#D55E00", "#CC79A7"), 
      # use 'scale_fill_viridis_c' for the following options
      "viridis" = pal, 
      "magma" = pal, 
      "inferno" = pal, 
      "plasma" = pal, 
      "viridis" = pal, 
      "cividis" = pal, 
      "rocket" = pal, 
      "mako" = pal, 
      "turbo" = pal, 
      "rainbow" = pal, 
      # for a single color name, combine with "gray95" for continuous color scale
      c("gray95", pal)
    )
  }
  
  # if length(pal) == 0 (i.e. 'pal' is NULL), leave 'pal' unchanged and the
  # plotting functions will select default palettes instead
  
  # if length(pal) > 1, use 'pal' as provided (e.g. multiple colors for discrete
  # labels, or length 2 for continuous gradient)
  
  return(pal)
}


#' @importFrom SpatialExperiment imgData
.sub_imgData <- function(spe, sample_ids, image_ids) {
    .get_img_idx <- SpatialExperiment:::.get_img_idx
    if (is.null(image_ids)) {
        # default to first available image for each sample
        idx <- .get_img_idx(spe, TRUE, NULL)
    } else {
        if (length(image_ids) == 1) {
            idx <- .get_img_idx(spe, TRUE, image_ids)
        } else {
            stopifnot(length(image_ids) == length(sample_ids))
            idx <- mapply(s = sample_ids, i = image_ids,
                          function(s, i) .get_img_idx(spe, s, i))
        }
    }
    imgData(spe)[idx, ]
}


.y_reverse <- function(df, ix, y, img) {
    y_tmp <- df[ix, y]
    if (!is.null(img)) {
        y_tmp <- nrow(img) - y_tmp
    } else {
        y_tmp <- max(y_tmp) - y_tmp
    }
    df[ix, y] <- y_tmp
    return(df)
}

test_that("getting color palette works", {
  expect_length(.get_pal("libd_layer_colors"), 8)
  expect_length(.get_pal("Okabe-Ito"), 8)
  expect_length(.get_pal("navy"), 2)
  expect_length(.get_pal(c("gray90", "navy")), 2)
})

#' plotVisium
#' 
#' Plots for spatially resolved transcriptomics data from the 10x Genomics
#' Visium platform
#' 
#' Function to generate plots for spatially resolved transcriptomics datasets
#' from the 10x Genomics Visium spatially platform.
#' 
#' This function generates a plot for spot-based spatially resolved
#' transcriptomics data from the 10x Genomics Visium platform, with several
#' options available to adjust the plot type and style.
#' 
#' 
#' @param spe (SpatialExperiment) Input data object.
#' 
#' @param spots (logical) Whether to display spots (spatial barcodes) as points.
#'   Default = TRUE.
#' 
#' @param annotate (character) Column in \code{colData} to use to fill points by
#'   color. If \code{annotate} contains a numeric column (e.g. total UMI counts), a
#'   continuous color scale will be used. If \code{annotate} contains a factor (e.g.
#'   cluster labels), a discrete color scale will be used. Default = NULL.
#' 
#' @param highlight (character) Column in \code{colData} to use to highlight
#'   points by outlining them. For example, \code{in_tissue} will highlight
#'   spots overlapping with tissue. Default = NULL.
#' 
#' @param facets (character) Column in \code{colData} to use to facet plots,
#'   i.e. show multiple panels of plots. Default = "sample_id". Set to NULL to
#'   disable.
#' 
#' @param image (logical) Whether to show histology image as background. Default
#'   = TRUE.
#'   
#' @param zoom (logical) Whether to zoom to area of tissue containing spots.
#'   Default = FALSE
#'   
#' @param show_axes (logical) Whether to show axes and coordinates. Default =
#'   FALSE
#' 
#' @param assay (character) Name of assay data to use when \code{annotate} is in
#'   \code{rownames(spe)}. Should be one of \code{assayNames(spe)}.
#'   
#' @param trans Transformation to apply for continuous scales. Ignored unless
#'   \code{annotate} is numeric, e.g. feature expression. (See
#'   \code{?continuous_scale} from \code{ggplot2} for valid options.)
#' 
#' @param point_shape (numeric) Point shape. Default = 21, which gives a
#'   circular shape suitable for representing for example, a Visium spot or
#'   Xenium cell. A value of 22 gives a square shape suitable for Visium HD.
#' 
#' @param point_size (numeric) Point size. Default = 1.
#' 
#' @param legend_position Legend position for annotations. Options are "left",
#'   "right", "top", "bottom", and "none". Default = "right".
#' 
#' @param x_coord (character) Column in \code{spatialCoords} containing
#'   x-coordinates. Default = NULL, which selects the first column.
#' 
#' @param y_coord (character) Column in \code{spatialCoords} containing
#'   y-coordinates. Default = NULL, which selects the second column.
#' 
#' @param y_reverse (logical) Whether to reverse y coordinates, which is often 
#'   required for Visium data, depending on the orientation of the raw data.
#'   Default = TRUE.
#' 
#' @param sample_ids (character) Samples to show, if multiple samples are
#'   available. Default = NULL (show all samples).
#' 
#' @param image_ids (character) Images to show, if multiple images are
#'   available. Default = NULL (show all images).
#' 
#' @param pal (character) Color palette for points. Options for discrete
#'   labels are "libd_layer_colors", "Okabe-Ito", or a custom vector of hex
#'   color codes. Options for continuous values are "viridis", a single color
#'   name (e.g. "red", "navy", etc), or a vector of length two containing color
#'   names for each end of the scale. Default = "libd_layer_colors" for discrete
#'   data, and "viridis" for continuous data.
#' 
#' 
#' @return Returns a ggplot object. Additional plot elements can be added as
#'   ggplot elements (e.g. title, customized formatting, etc).
#' 
#' 
#' @importFrom SpatialExperiment spatialCoords spatialCoordsNames imgData
#'   'imgData<-' imgRaster scaleFactors
#' @importFrom SummarizedExperiment colData assayNames
#' @importFrom ggplot2 ggplot scale_fill_manual scale_fill_gradient
#'   scale_fill_gradientn scale_fill_viridis_c scale_color_identity
#'   scale_fill_identity facet_wrap labs guides guide_colorbar guide_legend
#'   theme_void element_text margin unit layer
#' @importFrom grid rasterGrob
#' @importFrom ggrepel geom_text_repel
#' @importFrom grDevices colorRampPalette
#' @importFrom RColorBrewer brewer.pal
#' @importFrom scales hue_pal
#' @importFrom methods is as
#' @importFrom stats setNames
#' 
#' @export
#' 
#' @author Helena L. Crowell, with modifications by Lukas M. Weber and Yixing E.
#'   Dong
#' 
#' @examples
#' library(STexampleData)
#' 
#' spe <- Visium_mouseCoronal()
#' colData(spe)$sum <- colSums(counts(spe))
#' 
#' # color by colData, highlight in-tissue spots
#' plotVisium(spe, annotate = "sum", highlight = "in_tissue")
#' 
#' # subset in-tissue spots
#' sub <- spe[, as.logical(colData(spe)$in_tissue)]
#' 
#' # color by feature counts, don't include image
#' rownames(sub) <- make.names(rowData(sub)$gene_name)
#' plotVisium(sub, annotate = "Gad2", assay = "counts")
#' 
plotVisium <- function(spe, 
                       spots = TRUE, annotate = NULL, highlight = NULL, 
                       facets = "sample_id", image = TRUE, zoom = FALSE, 
                       show_axes = FALSE, assay = "counts", trans = "identity", 
                       point_shape = 21, point_size = 1, 
                       legend_position = "right", 
                       x_coord = NULL, y_coord = NULL, y_reverse = TRUE, 
                       sample_ids = NULL, image_ids = NULL, pal = NULL) {
  
  # check validity of input arguments
  stopifnot(
    is(spe, "SpatialExperiment"), 
    is.logical(spots), length(spots) == 1, 
    is.logical(image), length(image) == 1, 
    is.logical(y_reverse), length(y_reverse) == 1)
  
  stopifnot(legend_position %in% c("left", "right", "top", "bottom", "none"))
  
  if (!is.null(annotate)) {
    stopifnot(is.character(annotate))
  }
  
  if(is.null(x_coord)) x_coord <- spatialCoordsNames(spe)[1]
  if(is.null(y_coord)) y_coord <- spatialCoordsNames(spe)[2]
  
  # set up data for plotting
  df <- cbind(data.frame(colData(spe), check.names = FALSE), 
              data.frame(spatialCoords(spe), check.names = FALSE))
  if (!is.null(annotate)) {
    # check validity of 'annotate' argument
    stopifnot(is.character(annotate), length(annotate) == 1)
    if (!annotate %in% c(names(df), rownames(spe))) {
      stop("'annotate' should be in rownames(spe) or names(colData(spe))")
    }
    # (optionally) add feature assay data to 'df'
    if (annotate %in% rownames(spe)) {
      stopifnot(
        is.character(assay), 
        length(grep(paste0("^", assay, "$"), assayNames(spe))) == 1)
      df[[annotate]] <- assay(spe, assay)[annotate, ]
    }
    if (is.numeric(df[[annotate]]) & is.null(pal)) {
      # for continuous feature, ensure length(pal) == 1 (instead of 0 if NULL)
      pal <- "rainbow"
    }
    # get color palette
    pal <- .get_pal(pal, df[[annotate]])
  } else {
    annotate <- "foo"
    df[[annotate]] <- "black"
  }
  
  if (is.null(sample_ids)) {
    # default to using all samples
    sample_ids <- unique(spe$sample_id)
  } else {
    # subset specified samples
    spe <- spe[, spe$sample_id %in% sample_ids]
  }
  
  # subset selected images
  img_df <- .sub_imgData(spe, sample_ids, image_ids)
  rownames(img_df) <- img_df$sample_id
  
  # construct image layers
  # note: images could also be plotted using 'annotation_custom()', 
  # however, this does not allow for faceting, so we instead 
  # construct a separate image layer for each sample
  if (image) {
    images <- lapply(sample_ids, function(s) {
      spi <- img_df[s, "data"]
      img <- imgRaster(spi[[1]])
      layer(
        data = data.frame(sample_id = s, check.names = FALSE), 
        inherit.aes = FALSE, 
        stat = "identity", 
        position = "identity", 
        geom = ggplot2::GeomCustomAnn, 
        params = list(
          grob = rasterGrob(img), 
          xmin = 0, xmax = ncol(img), 
          ymin = 0, ymax = nrow(img))
      )
    })
    img <- img_df$data[[1]]
    xlim <- c(0, ncol(img))
    ylim <- c(0, nrow(img))
    
    if (zoom) {
      xlim <- ylim <- NULL
    }
  } else {
    img <- NULL
    images <- xlim <- ylim <- NULL
  }
  
  # scale spatial coordinates
  for (s in sample_ids) {
    ix <- df$sample_id == s
    xy <- c(x_coord, y_coord)
    sf <- img_df[s, "scaleFactor"]
    df[ix, xy] <- sf * df[ix, xy]
    # reverse y coordinates to match orientation of images 
    # (sometimes required for Visium data)
    if (y_reverse) df <- .y_reverse(df, ix, y_coord, img)
  }
  
  # construct points and highlights
  if (spots) {
    # check whether 'annotate' is continuous (numeric) or discrete (factor)
    guide <- ifelse(is.numeric(df[[annotate]]), guide_colorbar, guide_legend)
    points <- list(
      guides(fill = guide(
        title = annotate, order = 1, override.aes = list(col = NA, size = 3))), 
      geom_point(shape = point_shape, size = point_size, stroke = 0.25, alpha = 0.8))
    if (!is.null(highlight)) {
      df$highlight <- as.factor(df[[highlight]])
      highlights <- list(
        scale_color_manual(highlight, values = c("#e0e0e0", "black")), 
        guides(col = guide_legend(override.aes = list(
          size = 2, stroke = 1, 
          col = c("#e0e0e0", "black")[seq_along(unique(df$highlight))]))))
    } else {
      df$highlight <- "transparent"
      highlights <- scale_color_identity()
    }
  } else {
    # this is required, else the image layer doesn't show
    points <- geom_point(col = "transparent")
    highlights <- NULL
  }
  
  # color scale
  scale <- if(annotate != "foo") {
    if (is.numeric(df[[annotate]])) {
      if (length(pal) == 1 && 
          pal %in% c("viridis", "magma", "inferno", "plasma", 
                     "cividis", "rocket", "mako", "turbo")) {
        scale_fill_viridis_c(trans = trans, option = pal)
      } else if (length(pal) == 1 && pal == "rainbow") {
        scale_fill_gradientn(
          colors = colorRampPalette(
            colors = rev(x = brewer.pal(n = 11, name = "Spectral")))(100), 
          trans = trans, 
          limits = c(min(df[[annotate]]), max(df[[annotate]])))
      } else {
        scale_fill_gradient(low = pal[1], high = pal[2], trans = trans)
      }
    } else if (is.factor(df[[annotate]])) {
      # for categorical feature, automate palette
      if (is.null(pal)) {
        scale_fill_manual(
          name = annotate, 
          values = hue_pal()(length(unique(df[[annotate]]))))
      } else if (!is.null(pal)) {
        scale_fill_manual(values = pal)
      }
    }
  } else {
    scale_fill_identity()
  }
  
  # display plot
  p <- ggplot(df, aes(get(x_coord), get(y_coord), 
                      fill = if (!is.null(annotate)) get(annotate) else NULL, 
                      col = get("highlight"))) + 
    labs(fill = annotate) + 
    images + points + highlights + scale + 
    coord_fixed(xlim = xlim, ylim = ylim) 
  
  if (show_axes) {
    p <- p + 
      theme_bw() + 
      theme(strip.text = element_text(margin = margin(0, 0, 0.5, 0, "lines"), 
                                      size = 12), 
            legend.position = legend_position) +
      labs(x = paste0("pxl_col_in_", img_df[s, "image_id"]), 
           y = paste0("pxl_col_in_", img_df[s, "image_id"]))
  } else {
    p <- p + 
      theme_void() + 
      theme(strip.text = element_text(margin = margin(0, 0, 0.5, 0, "lines"), 
                                      size = 12), 
            legend.position = legend_position)
  }
  
  p <- p + if (!is.null(facets) && length(sample_ids) > 1) facet_wrap(facets)
  
  p
}

#' plotDimRed
#' 
#' Plotting functions for spatial transcriptomics data.
#' 
#' Function to create reduced dimension plot (e.g. PCA or UMAP) with additional
#' optional annotations such as cluster labels, expression of a gene, or quality
#' control metrics.
#' 
#' 
#' @param spe Input data, assumed to be a \code{SpatialExperiment} or
#'   \code{SingleCellExperiment} object.
#' 
#' @param plot_type Type of reduced dimension plot. Possible options are "UMAP",
#'   "PCA", or any other set of reduced dimensions stored in the input object.
#'   Default = "UMAP".
#' 
#' @param annotate Variable to show as annotations. This may be discrete or
#'   continuous. For a discrete variable (e.g. cluster labels), this should be
#'   the name of a column in \code{colData} containing a character vector or
#'   factor. For a continuous variable (e.g. a gene name), this should be an
#'   entry in \code{feature_names}. Default = NULL.
#' 
#' @param feature_names Name of column in \code{rowData} containing names of
#'   continuous features to plot (e.g. gene names). For example, set to
#'   \code{feature_names = "gene_name"} if gene names are stored in a column
#'   named \code{"gene_name"}. This argument is used if \code{annotate} is a
#'   continuous variable. Default = NULL, in which case the row names of the
#'   input object will be used.
#' 
#' @param assay_name Name of \code{assay} in input object containing values to
#'   plot for a continuous variable. Default = "counts".
#' 
#' @param update_dimnames Whether to update column names of \code{reducedDims}
#'   to default values for plotting. Default = TRUE.
#' 
#' @param pal Color palette for annotations. Options for discrete values are
#'   "libd_layer_colors", "Okabe-Ito", or any vector of color names or hex
#'   values. For continuous values, provide a vector of length 2 for the low and
#'   high range, e.g. c("gray90", "navy").
#' 
#' @param point_size Point size. Default = 0.3.
#' 
#' @param legend_point_size Legend point size for discrete annotations. Default
#'   = 3.
#' 
#' @param text_by Column name of annotation labels to display over each cluster
#'   of points. This will usually be the same as \code{annotate}. Alternatively,
#'   another column may be used (e.g. with more readable classes or shorter
#'   strings). Only used for discrete \code{annotate}. Default = NULL.
#' 
#' @param text_by_size Text size for annotation labels over each cluster.
#'   Default = 5.
#' 
#' @param text_by_color Color name or hex code for annotation labels. Default =
#'   "black".
#' 
#' 
#' @return Returns a ggplot object, which may be further modified using ggplot
#'   functions.
#' 
#' 
#' @importFrom SingleCellExperiment reducedDimNames reducedDim 'reducedDim<-'
#' @importFrom SummarizedExperiment assay colData rowData
#' @importFrom grDevices colorRampPalette
#' @importFrom RColorBrewer brewer.pal
#' @importFrom scales hue_pal
#' @importFrom stats median
#' @importFrom ggrepel geom_text_repel
#' @importFrom ggplot2 ggplot geom_point theme_bw theme element_blank
#'   scale_color_viridis_c scale_color_gradientn scale_color_gradient
#'   scale_color_manual ggtitle labs guides aes .data
#' 
#' @export
#' 
#' @author Lukas M. Weber and Yixing E. Dong
#' 
#' @examples
#' library(STexampleData)
#' spe <- Visium_humanDLPFC()
#' 
#' # select spots over tissue
#' spe <- spe[, colData(spe)$in_tissue == 1]
#' 
#' # use small subset of data for this example
#' n <- 200
#' set.seed(123)
#' spe <- spe[, sample(seq_len(ncol(spe)), n)]
#' 
#' # calculate logcounts
#' library(scran)
#' spe <- logNormCounts(spe)
#' 
#' # identify top highly variable genes (HVGs)
#' is_mito <- grepl("(^MT-)|(^mt-)", rowData(spe)$gene_name)
#' spe <- spe[!is_mito, ]
#' dec <- modelGeneVar(spe)
#' top_hvgs <- getTopHVGs(dec, prop = 0.1)
#' 
#' # run dimensionality reduction
#' library(scater)
#' set.seed(123)
#' spe <- runPCA(spe, subset_row = top_hvgs)
#' set.seed(123)
#' spe <- runUMAP(spe, dimred = "PCA")
#' colnames(reducedDim(spe, "UMAP")) <- paste0("UMAP", 1:2)
#' 
#' # generate plot
#' plotDimRed(spe, plot_type = "UMAP", annotate = "ground_truth")
#' 
plotDimRed <- function(spe, plot_type = c("UMAP", "PCA"), 
                       annotate = NULL, feature_names = NULL, 
                       assay_name = "counts", 
                       update_dimnames = TRUE, 
                       pal = NULL, point_size = 0.3, 
                       legend_point_size = 3, 
                       text_by = NULL, text_by_size = 5, 
                       text_by_color = "black") {
  
  # check validity of arguments
  stopifnot(length(plot_type) == 1)
  stopifnot(plot_type %in% reducedDimNames(spe))
  
  # get names of continuous features
  if (is.null(feature_names)) {
    feature_nms <- rownames(spe)
  } else {
    stopifnot(ncol(rowData(spe)) > 0)
    stopifnot(feature_names %in% colnames(rowData(spe)))
    feature_nms <- rowData(spe)[, feature_names]
  }
  
  if (!is.null(annotate)) {
    stopifnot(is.character(annotate))
    if (!(annotate %in% c(colnames(colData(spe)), feature_nms))) {
      stop("'annotate' should be either (i) the name of a column in colData ", 
           "or (ii) an entry in either a column named 'feature_names' in ", 
           "rowData or the rownames of the input object")
    }
  }
  
  # update colnames of reducedDims for plotting
  if (update_dimnames) {
    colnames(reducedDim(spe, plot_type)) <- 
      paste0(plot_type, "_", seq_len(ncol(reducedDim(spe, plot_type))))
  }
  
  # data frame for plotting
  df <- cbind(data.frame(colData(spe), check.names = FALSE), 
              data.frame(reducedDim(spe, plot_type), check.names = FALSE))
  
  if (!is.null(annotate)) {
    # continuous annotation values
    if (annotate %in% feature_nms) {
      stopifnot(is.character(assay_name))
      ix <- which(feature_nms == annotate)
      df[[annotate]] <- assay(spe, assay_name)[ix, ]
    }
    # discrete annotation values
    if ((annotate %in% colnames(colData(spe))) && 
        (is.character(colData(spe)[, annotate]))) {
      df[[annotate]] <- as.factor(df[[annotate]])
    }
  }
  
  # color palettes
  if ((is.null(annotate) || is.numeric(df[[annotate]])) && is.null(pal)) {
    # for continuous values, change NULL to arbitrary color so length(pal) == 1
    pal <- "blue"
  }
  # accepts "libd_layer_colors" and "Okabe-Ito", or arbitrary color palette for NULL
  pal <- .get_pal(pal)
  
  dim_labels <- colnames(reducedDim(spe, plot_type))[1:2]
  x_label <- dim_labels[1]
  y_label <- dim_labels[2]
  
  
  # main plot
  
  p <- ggplot(df, aes(x = get(x_label), y = get(y_label), 
                      color = if (!is.null(annotate)) get(annotate) else NULL)) + 
    geom_point(size = point_size) + 
    labs(x = x_label, 
         y = y_label, 
         color = annotate) + 
    theme_bw() + 
    theme(panel.grid = element_blank())
  
  
  # additional plot formatting
  
  # color scale
  if (!is.null(annotate)) {
    scaling <- if (is.numeric(df[[annotate]])) {
      # continuous values
      if (length(pal) == 1 && 
          pal %in% c("viridis", "magma", "inferno", "plasma", "cividis", 
                     "rocket", "mako", "turbo")) {
        scale_color_viridis_c(option = pal)
      } else if (length(pal) == 1 && pal == "rainbow") {
        colors <- colorRampPalette(
          colors = rev(x = brewer.pal(n = 11, name = "Spectral")))(100)
        scale_color_gradientn(colors = colors, limits = range(df[[annotate]]))
      } else {
        scale_color_gradient(low = pal[1], high = pal[2])
      }
    } else if (is.factor(df[[annotate]]) | is.character(df[[annotate]])) {
      # discrete values
      if (is.null(pal)) {
        scale_color_manual(name = annotate, 
                           values = hue_pal()(length(unique(df[[annotate]]))))
      } else if (!is.null(pal)) {
        scale_color_manual(values = pal)
      }
    }
  }
  
  # plot title
  if (!is.null(annotate)) {
    if (is.numeric(df[[annotate]])) {
      # continuous values: display plot title but no legend title
      p <- p + 
        scaling + 
        ggtitle(annotate) + 
        labs(color = NULL) + 
        theme(plot.title = element_text(hjust = 0.5))
    } else if (is.factor(df[[annotate]]) | is.character(df[[annotate]])) {
      # discrete values: display legend title but no plot title
      p <- p + 
        scaling + 
        guides(color = guide_legend(override.aes = list(size = legend_point_size)))
    }
  }
  
  # text annotations
  if (!is.null(annotate)) {
    if (is.factor(df[[annotate]]) | is.character(df[[annotate]])) {
      # add text with the median locations of the 'text_by' vector
      if (!is.null(text_by)) {
        by_text_x <- vapply(
          split(df[[x_label]], df[[text_by]]), 
          median, 
          FUN.VALUE = 0
        )
        by_text_y <- vapply(
          split(df[[y_label]], df[[text_by]]), 
          median, 
          FUN.VALUE = 0
        )
        p <- p + 
          geom_text_repel(
            data = data.frame(
              x = by_text_x, 
              y = by_text_y, 
              label = names(by_text_x), 
              check.names = FALSE
            ), 
            mapping = aes(x = .data$x, y = .data$y, label = .data$label), 
            size = text_by_size, 
            color = text_by_color
          )
      }
    }
  }
  
  # return plot
  p
}

#' plotFeatureQC
#' 
#' Plotting functions for spatial transcriptomics data.
#' 
#' Function to create feature-level quality control (QC) plots for spatial
#' transcriptomics data.
#' 
#' The following types of feature-level (i.e. gene-level) QC plots are
#' available:
#' 
#' \itemize{
#' \item Histogram (\code{plot_type = "histogram"}) for a single QC metric, e.g.
#' total UMI counts across all spots per feature. The histogram can optionally
#' highlight selected features, e.g. low abundance features.
#' \item Violin (\code{plot_type = "violin"}) for a single QC metric, e.g. total
#' UMI counts across all spots per feature. The violin plot can optionally
#' highlight selected features, e.g. low abundance features.
#' }
#' 
#' For observation-level (i.e. spot-level or cell-level) QC plots, see
#' \code{\link{plotObsQC}}.
#' 
#' 
#' @param spe Input data, assumed to be a \code{SpatialExperiment} or
#'   \code{SingleCellExperiment} object.
#' 
#' @param plot_type Type of QC plot. Options are "histogram" and "violin". See
#'   Details for additional details.
#' 
#' @param x_metric Name of column in \code{rowData} containing feature-level QC
#'   metric to plot on x-axis. Required for histograms and violin plots.
#' 
#' @param annotate Name of column in \code{rowData} identifying selected
#'   features that do not meet QC filtering thresholds, which will be
#'   highlighted on a histogram or violin plot. Default = NULL. Optional
#'   argument used for histograms and violin plots.
#' 
#' @param n_bins Number of bins for histograms. Default = 100. Optional argument
#'   used for histograms.
#' 
#' @param point_size Point size. Default = 0.1. Optional argument for violin
#'   plots.
#' 
#' @param scale_log1p Whether to log1p-scale axes. Default = TRUE.
#' 
#' 
#' @return Returns a ggplot object, which may be further modified using ggplot
#'   functions.
#' 
#' 
#' @importFrom SummarizedExperiment rowData
#' @importFrom ggplot2 ggplot geom_histogram geom_violin geom_jitter
#'   scale_fill_manual scale_color_manual scale_x_continuous scale_y_continuous
#'   xlab labs theme_bw theme element_blank
#' 
#' 
#' @export
#' 
#' @author Yixing E. Dong and Lukas M. Weber
#' 
#' @examples
#' library(STexampleData)
#' spe <- Visium_humanDLPFC()
#' 
#' rowData(spe)$feature_sum <- rowSums(counts(spe))
#' rowData(spe)$low_abundance <- rowSums(counts(spe) > 0) < 20
#' 
#' plotFeatureQC(spe, plot_type = "histogram", 
#'               x_metric = "feature_sum", annotate = "low_abundance")
#' plotFeatureQC(spe, plot_type = "violin", 
#'               x_metric = "feature_sum", annotate = "low_abundance")
#' 
plotFeatureQC <- function(spe, plot_type = c("histogram", "violin"), 
                          x_metric = NULL, annotate = NULL, 
                          n_bins = 100, point_size = 0.1, 
                          scale_log1p = TRUE) {
  
  # check validity of arguments
  plot_type <- match.arg(plot_type)
  
  df <- data.frame(rowData(spe))
  
  # for histogram or violin plots
  if (!is.null(annotate)) {
    stopifnot(is.character(annotate))
    if (!is.logical(df[[annotate]])) {
      stop("'annotate' should be a vector of logical values in rowData")
    }
  }
  
  
  # histogram: requires 'x_metric' (continuous), optionally 'annotate' (logical)
  
  if (plot_type == "histogram") {
    
    stopifnot(is.numeric(n_bins))
    
    # histogram showing 'x_metric', optionally colored by 'annotate'
    if (!is.null(annotate)) {
      p <- ggplot(df, aes(x = get(x_metric), fill = get(annotate))) + 
        geom_histogram(bins = n_bins, color = "#e9ecef", alpha = 0.6, 
                       position = "identity") + 
        scale_fill_manual(values = c("gray70", "red")) + 
        labs(fill = annotate)
    } else if (is.null(annotate)) {
      p <- ggplot(df, aes(x = get(x_metric))) + 
        geom_histogram(bins = n_bins, color = "#e9ecef", alpha = 0.6, 
                       position = "identity") + 
        scale_fill_manual(values = c("gray70"))
    }
    
    p <- p + xlab(x_metric)
    
    if (scale_log1p) {
      p <- p + scale_x_continuous(transform = "log1p")
    }
    
    p <- p + theme_bw()
  }
  
  
  # violin: requires 'x_metric' (continuous), optionally 'annotate' (logical)
  
  if (plot_type == "violin") {
    
    df[["dummy"]] <- rep(" ", nrow(df))
    
    p <- ggplot(df, aes(x = get("dummy"), y = get(x_metric), fill = get("dummy"))) + 
      geom_violin(trim = TRUE, alpha = 0.9) + 
      scale_fill_manual(values = c("gray70")) + 
      labs(x = "dummy", 
           y = x_metric, 
           fill = "dummy") + 
      theme_bw() + 
      theme(legend.position = "none", 
            panel.grid = element_blank())
    
    if (is.null(annotate)) {
      # violins for 'x_metric'
      p <- p + 
        geom_jitter(size = point_size)
    } else if (!is.null(annotate)) {
      # violins for 'x_metric', colored by 'annotate' (colors in order FALSE, TRUE)
      p <- p + 
        geom_jitter(aes(color = get(annotate)), size = point_size) + 
        scale_color_manual(values = c("black", "red")) + 
        labs(color = annotate)
    }
    
    if (scale_log1p) {
      p <- p + scale_y_continuous(transform = "log1p")
    }
  }
  
  # return plot
  p
}

---
title: "ggspavis overview"
author: 
  - name: Lukas M. Weber
    affiliation: "Boston University, Boston, MA, USA"
  - name: Helena L. Crowell
    affiliation: "University of Zurich, Zurich, Switzerland"
  - name: Yixing E. Dong
    affiliation: "University of Lausanne, Lausanne, Switzerland"
package: ggspavis
output: 
  BiocStyle::html_document
vignette: >
  %\VignetteIndexEntry{ggspavis overview}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
```

# Introduction

The `ggspavis` package contains a set of visualization functions for spatial transcriptomics data, designed to work with the [SpatialExperiment](https://bioconductor.org/packages/SpatialExperiment) Bioconductor object class.

# Examples

Load some example datasets from the [STexampleData](https://bioconductor.org/packages/STexampleData) or [spatialLIBD](https://research.libd.org/spatialLIBD/) package and create some example plots to demonstrate `ggspavis`.

```{r, message=FALSE, warning=FALSE}
library(ggspavis)
library(STexampleData)
library(patchwork)
library(scater)
library(scran)
library(OSTA.data)
library(VisiumIO)
library(SpatialExperiment)
```

## Sequencing-based spatial transcriptomics data

### Spot shape - Slide-seq and Visium

First, we start with a demo for Slide-seq V2 mouse brain dataset.
```{r loadslide, message=FALSE, warning=FALSE}
spe_slide <- STexampleData::SlideSeqV2_mouseHPC()
spe_slide$loglibsize <- log1p(colSums(counts(spe_slide)))
```

We can visualize the barcoded beads of Slide-seq V2 spatially with `plotCoords()`.
```{r pltcoordsslide, fig.width=10, fig.height=3, warning=FALSE, message=FALSE}
(plotCoords(spe_slide, annotate = "celltype", 
            in_tissue = NULL, point_size = 0.1) + 
  guides(color=guide_legend(override.aes = list(size = 3), ncol = 2)) |
  plotCoords(spe_slide, annotate = "loglibsize", 
             in_tissue = NULL, point_size = 0.1) + 
  scale_color_gradient(name = "Log library size") + ggtitle("")) +
  plot_annotation(title = 'Slide-seq V2 Mouse Brain')
```

Similarly, in a 10x Genomics Visium mouse brain dataset, we generate visualizations of library size and expression levels of selected genes. Both `plotVisium()` and `plotCoords()` reflect the spatial coordinates of spots, with the former also overlaying spots on the H\&E histology image. Note that `plotVisium()` accepts a `SpatialExperiment` class object (with image data), while other functions in the package accept either `SpatialExperiment` or `SingleCellExperiment` class objects.

```{r loadvismb, message=FALSE}
# load data in SpatialExperiment format
spe_vm <- Visium_mouseCoronal()
rownames(spe_vm) <- rowData(spe_vm)$gene_name
colData(spe_vm)$sum <- colSums(counts(spe_vm))
```

With `plotVisium()` annotated by a continuous variable, you can adjust palette, legend position, scaling of the variable, and whether to highlight spots that are in tissue, etc.

```{r vismbvislibsize, message=FALSE, warning=FALSE, fig.width=8, fig.height=3.5}
p1 <- plotVisium(spe_vm, annotate = "sum", highlight = "in_tissue", 
                 legend_position = "none")
p2 <- plotVisium(spe_vm, annotate = "sum", highlight = "in_tissue", 
                 pal = "darkred") + 
  guides(fill = guide_colorbar(title = "Libsize"))

# display panels using patchwork
p1 | p2
```

`plotVisium()` can also be used to visualize gene expression.

```{r vismbgevisge, warning=FALSE, fig.width=8, fig.height=3.5}
p1 <- plotVisium(spe_vm, annotate = "Gapdh", highlight = "in_tissue")
p2 <- plotVisium(spe_vm, annotate = "Mbp", highlight = "in_tissue")

# display panels using patchwork
p1 | p2
```

Two other possibilities with `plotVisium()` are to show only spots or only the H\&E image.

```{r vismbonlyspots, warning=FALSE, fig.width=8, fig.height=3.5}
p1 <- plotVisium(spe_vm, annotate = "Mbp", 
                 highlight = "in_tissue", image = FALSE)
p2 <- plotVisium(spe_vm, annotate = "Mbp", 
                 highlight = "in_tissue", spots = FALSE)

# display panels using patchwork
p1 | p2
```

`plotCoords()` by default subsets to only spots that are in tissue for a 10x Genomics Visium dataset. You can either leave the palette as NULL for a continuous, or change the palette in `plotCoords()` in a similar manner as in `plotVisium()`.

```{r vismbcoordsge, fig.width=6, fig.height=3}
p1 <- plotCoords(spe_vm, annotate = "Gapdh")
p2 <- plotCoords(spe_vm, annotate = "Mbp", pal = "viridis")

# display panels using patchwork
p1 | p2
```

`plotCoords()` and `plotVisium()` can also be used to visualize discrete or categorical annotation variables, such as cluster labels as colors on the spatial coordinates. We will introduce this functionality using the Visium human brain dorsolateral prefrontal cortex (DLPFC) dataset.

```{r loaddlpfc, message=FALSE}
# load data in SpatialExperiment format
spe <- Visium_humanDLPFC()
rownames(spe) <- rowData(spe)$gene_name
colData(spe)$libsize <- colSums(counts(spe))
```

First, we check the manually annotated reference labels, highlighting the spots that are in tissue, using `plotVisium()`.

```{r dlpfcgroundtruth, message=FALSE, out.width="60%"}
plotVisium(spe, annotate = "ground_truth", highlight = "in_tissue", 
           pal = "libd_layer_colors")
```

Here are some other choices of palettes. 

```{r testpalettes, fig.width=6, fig.height=3}
p1 <- plotCoords(spe, annotate = "ground_truth", pal = "Okabe-Ito") +
  ggtitle("Reference")
p2 <- plotCoords(spe, annotate = "libsize", pal = "rainbow") + 
  ggtitle("Library size")

# display panels using patchwork
p1 | p2
```

### Quality control (QC) plots

Note that these QC plot functions can be used for any `SpatialExperiment` object, not just Visium. For demonstration, we keep using Visium DLPFC dataset.

#### Spot/bin/cell-level QC

We next derive some spot-level quality control (QC) flags for plotting. We use the `scater` package to add QC metrics to our data object. 

```{r qcmetrics, message=FALSE}
# calculate QC metrics using scater
spe <- addPerCellQCMetrics(spe, 
  subsets = list(mito = grepl("(^MT-)|(^mt-)", rowData(spe)$gene_name)))

# apply QC thresholds
colData(spe)$low_libsize <- colData(spe)$sum < 400 | colData(spe)$detected < 400
colData(spe)$high_mito <- colData(spe)$subsets_mito_percent > 30
```

`plotObsQC(plot_type = "spot")` reflects the spatial coordinates of the spots, where spots of interests can be labeled by a flag with TRUE or FALSE levels. The TRUE level are highlighted by red color.

We can investigate spots with low library size using histograms, violin plots, and spot plots.

```{r obsqclowlibsize, fig.width=8, fig.height=2.5}
p1 <- plotObsQC(spe, plot_type = "histogram", 
                x_metric = "sum", annotate = "low_libsize")
p2 <- plotObsQC(spe, plot_type = "violin", 
                x_metric = "sum", annotate = "low_libsize", point_size = 0.1)
p3 <- plotObsQC(spe, plot_type = "spot", in_tissue = "in_tissue", 
                annotate = "low_libsize", point_size = 0.2)

# display panels using patchwork
p1 | p2 | p3
```

Similarly, we can investigate spots with high mitochondrial proportion of reads.

```{r obsqchighmt, fig.width=8, fig.height=2.5}
p1 <- plotObsQC(spe, plot_type = "histogram", 
                x_metric = "subsets_mito_percent", annotate = "high_mito")
p2 <- plotObsQC(spe, plot_type = "violin", 
                x_metric = "subsets_mito_percent", annotate = "high_mito", 
                point_size = 0.1)
p3 <- plotObsQC(spe, plot_type = "spot", in_tissue = "in_tissue", 
                annotate = "high_mito", point_size = 0.2)

# display panels using patchwork
p1 | p2 | p3
```

We can also use a scatter plot to check the trend between two variables, for example mitochondrial proportion vs. library size. We can also highlight spots by putting thresholds on the x and/or y axes.

```{r obsqcscatter, out.width="60%", warning=FALSE, message=FALSE}
plotObsQC(spe, plot_type = "scatter", 
          x_metric = "subsets_mito_percent", y_metric = "sum", 
          x_threshold = 30, y_threshold = 400)
```

#### Feature-level QC

Perform feature-level (gene-level) QC and visualize the result with a histogram. For example, for Visium, we demonstrate an arbitrary threshold that a gene should be detected in at least 20 spots to be considered not lowly abundant. The plot includes `log1p` transformation for easier visualization.

```{r featqc, warning=FALSE, message=FALSE, fig.width=8, fig.height=3}
rowData(spe)$feature_sum <- rowSums(counts(spe))
rowData(spe)$low_abundance <- rowSums(counts(spe) > 0) < 20

p1 <- plotFeatureQC(spe, plot_type = "histogram", 
                    x_metric = "feature_sum", annotate = "low_abundance")
p2 <- plotFeatureQC(spe, plot_type = "violin", 
                    x_metric = "feature_sum", annotate = "low_abundance")

# display panels using patchwork
p1 | p2
```

### Bin shape - Visium HD

Visium HD contains data binned into square shapes. We load an example dataset at 8 $\mu$m, and subset to a smaller region. 

```{r loadvhd8sub, message=FALSE}
# retrieve dataset from OSF repo
id <- "VisiumHD_HumanColon_Oliveira"
pa <- OSTA.data_load(id)
dir.create(td <- tempfile())
unzip(pa, exdir=td)

# read 8um bins into 'SpatialExperiment'
vhd8 <- TENxVisiumHD(spacerangerOut=td, processing="filtered", format="h5", 
                     images="lowres", bin_size="008") |> import()
# subset
vhd8 <- vhd8[, spatialCoords(vhd8)[, 1] * scaleFactors(vhd8) > 430 & 
               spatialCoords(vhd8)[, 1] * scaleFactors(vhd8) < 435 &
               spatialCoords(vhd8)[, 2] * scaleFactors(vhd8) > 127 & 
               spatialCoords(vhd8)[, 2] * scaleFactors(vhd8) < 132]
rownames(vhd8) <- rowData(vhd8)$Symbol
vhd8
```

Similar to Visium, we can also plot the data points spatially, with or without the H\&E image overlayed. However, we need to change the point shape to square. The default value for `point_shape` is `16` in `plotCoords()` and `21` in `plotVisium()` for Visium spots. These values should be updated to `15` and `22` for Visium HD bins, respectively. Let us visualize the gene expression of *PIGR*, with or without H\&E. 
```{r vhd8coord, warning=FALSE, fig.width=5, fig.height=2.5}
plotCoords(vhd8, point_shape=15, point_size = 1.7, annotate="PIGR") | 
  plotVisium(vhd8, point_shape=22, point_size = 2, annotate="PIGR", 
           zoom = TRUE)
```

## Imaging-based spatial transcriptomics data

We load example datasets from Xenium (10x Genomics), CosMx (Nanostring, now Bruker), MERSCOPE (Vizgen), and STARmapPLUS. 

```{r imgdataload, message=FALSE, warning=FALSE}
spe_xen <- STexampleData::Janesick_breastCancer_Xenium_rep1()
spe_cos <- STexampleData::CosMx_lungCancer()
spe_mer <- STexampleData::MERSCOPE_ovarianCancer()
spe_sta <- STexampleData::STARmapPLUS_mouseBrain()
```

Note that `in_tissue = NULL` must be specified for all imaging-based SPE objects. Point size could be adjusted manually if needed.

```{r imgspatial, message=FALSE, fig.width=12, fig.height=3}
plotCoords(spe_xen, annotate = "cell_area", in_tissue = NULL, pal = "magma",
           point_size = 0.15) + ggtitle("Xenium Breast Cancer") |
  plotCoords(spe_cos, annotate = "Area", in_tissue = NULL, pal = "plasma", 
             point_size = 0.1) + ggtitle("CosMx Breast Cancer") |
  plotCoords(spe_mer, annotate = "volume", in_tissue = NULL, 
             pal = c("navyblue", "yellow"),
             point_size = 0.05) + ggtitle("MERSCOPE Ovarian Cancer")
```

Another imaging-based technology STARmapPLUS has annotated mouse brain data. We demonstrate the following visualization on this relatively small dataset. Note, we can overlay text labels over the clusters using the `text_by` argument.

```{r spatialtextby, message=FALSE, fig.width=6, fig.height=3.5}
plotCoords(spe_sta, annotate = "Main_molecular_tissue_region", in_tissue = NULL,
           point_size = 0.1, text_by = "Main_molecular_tissue_region",
           text_by_size = 4, text_by_color = "#2d2d2d")
```

Here we perform some quick processing to get reduced dimensions.

```{r getumap}
spe_sta <- logNormCounts(spe_sta)
dec <- modelGeneVar(spe_sta)
hvg <- getTopHVGs(dec, n=3e3)
spe_sta <- runPCA(spe_sta, subset_row=hvg)
spe_sta <- runUMAP(spe_sta, dimred="PCA")
```

## Reduced dimension plots

We can also use the `plotDimRed()` function to generate reduced dimension plots, e.g. PCA or UMAP, with on-cluster annotation (at the center location of each cluster) using `text_by`.

```{r pltdimred, message=FALSE, fig.width=6, fig.height=3.5}
plotDimRed(spe_sta, plot_type = "UMAP",
           annotate = "Main_molecular_tissue_region", 
           text_by = "Main_molecular_tissue_region",
           text_by_size = 3, text_by_color = "#2d2d2d")
```

# Session information

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotMolecules.R
\name{plotMolecules}
\alias{plotMolecules}
\title{plotMolecules}
\usage{
plotMolecules(
  spe,
  molecule = NULL,
  x_coord = NULL,
  y_coord = NULL,
  sample_id = "sample_id",
  pal = c("gray90", "navy"),
  point_size = 0.3
)
}
\arguments{
\item{spe}{(SpatialExperiment) Input data, assumed to be a
\code{SpatialExperiment} object.}

\item{molecule}{Name of mRNA molecule to plot (assumed to match one of the
row names of \code{rowData}).}

\item{x_coord}{Name of column in \code{spatialCoords} containing x
coordinates. Default = NULL, which selects the first column of
\code{spatialCoords}.}

\item{y_coord}{Name of column in \code{spatialCoords} containing y
coordinates. Default = NULL, which selects the second column of
\code{spatialCoords}.}

\item{sample_id}{Name of column in \code{colData} containing sample IDs. This
argument is only required for datasets containing multiple samples (tissue
sections). If provided, samples will be shown in multiple panels using
facetting. Default = NULL.}

\item{pal}{Color palette, provided as a vector of length 2 for the low and
high range. Default = c("gray90", "navy").}

\item{point_size}{Point size. Default = 0.3.}
}
\value{
Returns a ggplot object, which may be further modified using ggplot
  functions.
}
\description{
Plotting functions for spatial transcriptomics data.
}
\details{
Function to create spot plot for molecule-based datasets, showing spatial
locations in x-y coordinates with optional annotations such as expression of
a gene.
}
\examples{
library(STexampleData)
spe <- seqFISH_mouseEmbryo()
plotMolecules(spe, molecule = "Sox2")

}
\author{
Lukas M. Weber
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotFeatureQC.R
\name{plotFeatureQC}
\alias{plotFeatureQC}
\title{plotFeatureQC}
\usage{
plotFeatureQC(
  spe,
  plot_type = c("histogram", "violin"),
  x_metric = NULL,
  annotate = NULL,
  n_bins = 100,
  point_size = 0.1,
  scale_log1p = TRUE
)
}
\arguments{
\item{spe}{Input data, assumed to be a \code{SpatialExperiment} or
\code{SingleCellExperiment} object.}

\item{plot_type}{Type of QC plot. Options are "histogram" and "violin". See
Details for additional details.}

\item{x_metric}{Name of column in \code{rowData} containing feature-level QC
metric to plot on x-axis. Required for histograms and violin plots.}

\item{annotate}{Name of column in \code{rowData} identifying selected
features that do not meet QC filtering thresholds, which will be
highlighted on a histogram or violin plot. Default = NULL. Optional
argument used for histograms and violin plots.}

\item{n_bins}{Number of bins for histograms. Default = 100. Optional argument
used for histograms.}

\item{point_size}{Point size. Default = 0.1. Optional argument for violin
plots.}

\item{scale_log1p}{Whether to log1p-scale axes. Default = TRUE.}
}
\value{
Returns a ggplot object, which may be further modified using ggplot
  functions.
}
\description{
Plotting functions for spatial transcriptomics data.
}
\details{
Function to create feature-level quality control (QC) plots for spatial
transcriptomics data.

The following types of feature-level (i.e. gene-level) QC plots are
available:

\itemize{
\item Histogram (\code{plot_type = "histogram"}) for a single QC metric, e.g.
total UMI counts across all spots per feature. The histogram can optionally
highlight selected features, e.g. low abundance features.
\item Violin (\code{plot_type = "violin"}) for a single QC metric, e.g. total
UMI counts across all spots per feature. The violin plot can optionally
highlight selected features, e.g. low abundance features.
}

For observation-level (i.e. spot-level or cell-level) QC plots, see
\code{\link{plotObsQC}}.
}
\examples{
library(STexampleData)
spe <- Visium_humanDLPFC()

rowData(spe)$feature_sum <- rowSums(counts(spe))
rowData(spe)$low_abundance <- rowSums(counts(spe) > 0) < 20

plotFeatureQC(spe, plot_type = "histogram", 
              x_metric = "feature_sum", annotate = "low_abundance")
plotFeatureQC(spe, plot_type = "violin", 
              x_metric = "feature_sum", annotate = "low_abundance")

}
\author{
Yixing E. Dong and Lukas M. Weber
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotObsQC.R
\name{plotObsQC}
\alias{plotObsQC}
\alias{plotQC}
\alias{plotSpotQC}
\title{plotObsQC}
\usage{
plotObsQC(
  spe,
  plot_type = c("histogram", "scatter", "spot", "violin"),
  x_coord = NULL,
  y_coord = NULL,
  x_metric = NULL,
  y_metric = NULL,
  x_threshold = NULL,
  y_threshold = NULL,
  trend = TRUE,
  marginal = TRUE,
  annotate = NULL,
  in_tissue = NULL,
  legend_point_size = 3,
  n_bins = 100,
  point_size = 0.3,
  y_reverse = TRUE
)

plotQC(...)

plotSpotQC(...)
}
\arguments{
\item{spe}{Input data, assumed to be a \code{SpatialExperiment} or
\code{SingleCellExperiment} object.}

\item{plot_type}{Type of QC plot. Options are "histogram", "scatter", "spot",
and "violin". See Details for additional details.}

\item{x_coord}{Name of column in \code{spatialCoords} (for a
\code{SpatialExperiment} input object) or \code{colData} (for a
\code{SingleCellExperiment} input object) containing x coordinates. Default
= NULL (for a \code{SpatialExperiment}, the first column of
\code{spatialCoords} will be selected in this case). Used for spot plots.}

\item{y_coord}{Name of column in \code{spatialCoords} (for a
\code{SpatialExperiment} input object) or \code{colData} (for a
\code{SingleCellExperiment} input object) containing y coordinates. Default
= NULL (for a \code{SpatialExperiment}, the second column of
\code{spatialCoords} will be selected in this case). Used for spot plots.}

\item{x_metric}{Name of column in \code{colData} containing QC metric to plot
on x-axis. Required for histograms, scatter plots, and violin plots.}

\item{y_metric}{Name of column in \code{colData} containing QC metric to plot
on y-axis. Required for histograms, scatter plots, and violin plots.}

\item{x_threshold}{QC filtering threshold on x-axis metric to highlight with
vertical line. Default = NULL. Optional argument used for scatter plots.}

\item{y_threshold}{QC filtering threshold on y-axis metric to highlight with
horizontal line. Default = NULL. Optional argument used for scatter plots.}

\item{trend}{Whether to show smoothed trend line (loess). Default = TRUE.
Optional argument used for scatter plots.}

\item{marginal}{Whether to show marginal histograms. Default = TRUE. Optional
argument used for scatter plots.}

\item{annotate}{Name of column in \code{colData} identifying selected spots
that do not meet QC filtering thresholds, which will be highlighted on a
histogram, spot plot, or violin plot. Default = NULL. Optional argument
used for histograms, spot plots, and violin plots.}

\item{in_tissue}{Name of column in \code{colData} identifying spots over
tissue (e.g. "in_tissue" for 10x Genomics Visium datasets). If this
argument is provided, only spots over tissue will be shown. Default = NULL.
Optional argument used for spot plots.}

\item{legend_point_size}{Legend point size. Default = 3. Optional argument
used for spot plots.}

\item{n_bins}{Number of bins for histograms. Default = 100. Optional argument
used for histograms.}

\item{point_size}{Point size. Default = 0.3. Optional argument for scatter
plots, spot plots, and violin plots. Suggested values: 0.5 for scatter
plots, 0.3 for spot plots, 0.1 for violin plots.}

\item{y_reverse}{Whether to reverse y coordinates. This is usually required
for 10x Genomics Visium datasets when using the default coordinate values.
Default = TRUE. Set to FALSE if not needed, e.g. for other platforms.
Optional argument used for spot plots.}

\item{...}{Not used.}
}
\value{
Returns a ggplot object, which may be further modified using ggplot
  functions.
}
\description{
Plotting functions for spatial transcriptomics data.
}
\details{
Function to create observation-level quality control (QC) plots for spatial
transcriptomics data.

The following types of observation-level (i.e. spot-level or cell-level) QC
plots are available:

\itemize{
\item Histogram (\code{plot_type = "histogram"}) for a single QC metric, e.g.
number of UMI counts per spot. For number of counts per spot, the histogram
can optionally highlight selected spots, e.g. spots with low library size.
\item Scatter plot (\code{plot_type = "scatter"}) comparing two QC metrics,
e.g. number of detected features vs. number of cells per spot, with optional
horizontal and vertical lines highlighting QC filtering thresholds.
\item Spot plot (\code{plot_type = "spot"}) showing spots in spatial x-y
coordinates, e.g. highlighting selected spots that do not meet filtering
thresholds.
\item Violin plot (\code{plot_type = "violin"}) for a single QC metric, e.g.
number of UMI counts per spot. For number of counts per spot, the violin plot
can optionally highlight selected spots, e.g. spots with low library size.
}

For feature-level (i.e. gene-level) QC plots, see
\code{\link{plotFeatureQC}}.
}
\examples{
library(STexampleData)
spe <- Visium_humanDLPFC()

colData(spe)$sum <- colSums(counts(spe))
colData(spe)$low_libsize <- colData(spe)$sum < 400

plotObsQC(spe, plot_type = "histogram", x_metric = "sum", annotate = "low_libsize")
plotObsQC(spe, plot_type = "scatter", x_metric = "sum", y_metric = "cell_count")
plotObsQC(spe, plot_type = "spot", annotate = "low_libsize", in_tissue = "in_tissue")
plotObsQC(spe, plot_type = "violin", x_metric = "sum", annotate = "low_libsize")

}
\author{
Lukas M. Weber and Yixing E. Dong
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotVisium.R
\name{plotVisium}
\alias{plotVisium}
\title{plotVisium}
\usage{
plotVisium(
  spe,
  spots = TRUE,
  annotate = NULL,
  highlight = NULL,
  facets = "sample_id",
  image = TRUE,
  zoom = FALSE,
  show_axes = FALSE,
  assay = "counts",
  trans = "identity",
  point_shape = 21,
  point_size = 1,
  legend_position = "right",
  x_coord = NULL,
  y_coord = NULL,
  y_reverse = TRUE,
  sample_ids = NULL,
  image_ids = NULL,
  pal = NULL
)
}
\arguments{
\item{spe}{(SpatialExperiment) Input data object.}

\item{spots}{(logical) Whether to display spots (spatial barcodes) as points.
Default = TRUE.}

\item{annotate}{(character) Column in \code{colData} to use to fill points by
color. If \code{annotate} contains a numeric column (e.g. total UMI counts), a
continuous color scale will be used. If \code{annotate} contains a factor (e.g.
cluster labels), a discrete color scale will be used. Default = NULL.}

\item{highlight}{(character) Column in \code{colData} to use to highlight
points by outlining them. For example, \code{in_tissue} will highlight
spots overlapping with tissue. Default = NULL.}

\item{facets}{(character) Column in \code{colData} to use to facet plots,
i.e. show multiple panels of plots. Default = "sample_id". Set to NULL to
disable.}

\item{image}{(logical) Whether to show histology image as background. Default
= TRUE.}

\item{zoom}{(logical) Whether to zoom to area of tissue containing spots.
Default = FALSE}

\item{show_axes}{(logical) Whether to show axes and coordinates. Default =
FALSE}

\item{assay}{(character) Name of assay data to use when \code{annotate} is in
\code{rownames(spe)}. Should be one of \code{assayNames(spe)}.}

\item{trans}{Transformation to apply for continuous scales. Ignored unless
\code{annotate} is numeric, e.g. feature expression. (See
\code{?continuous_scale} from \code{ggplot2} for valid options.)}

\item{point_shape}{(numeric) Point shape. Default = 21, which gives a
circular shape suitable for representing for example, a Visium spot or
Xenium cell. A value of 22 gives a square shape suitable for Visium HD.}

\item{point_size}{(numeric) Point size. Default = 1.}

\item{legend_position}{Legend position for annotations. Options are "left",
"right", "top", "bottom", and "none". Default = "right".}

\item{x_coord}{(character) Column in \code{spatialCoords} containing
x-coordinates. Default = NULL, which selects the first column.}

\item{y_coord}{(character) Column in \code{spatialCoords} containing
y-coordinates. Default = NULL, which selects the second column.}

\item{y_reverse}{(logical) Whether to reverse y coordinates, which is often 
required for Visium data, depending on the orientation of the raw data.
Default = TRUE.}

\item{sample_ids}{(character) Samples to show, if multiple samples are
available. Default = NULL (show all samples).}

\item{image_ids}{(character) Images to show, if multiple images are
available. Default = NULL (show all images).}

\item{pal}{(character) Color palette for points. Options for discrete
labels are "libd_layer_colors", "Okabe-Ito", or a custom vector of hex
color codes. Options for continuous values are "viridis", a single color
name (e.g. "red", "navy", etc), or a vector of length two containing color
names for each end of the scale. Default = "libd_layer_colors" for discrete
data, and "viridis" for continuous data.}
}
\value{
Returns a ggplot object. Additional plot elements can be added as
  ggplot elements (e.g. title, customized formatting, etc).
}
\description{
Plots for spatially resolved transcriptomics data from the 10x Genomics
Visium platform
}
\details{
Function to generate plots for spatially resolved transcriptomics datasets
from the 10x Genomics Visium spatially platform.

This function generates a plot for spot-based spatially resolved
transcriptomics data from the 10x Genomics Visium platform, with several
options available to adjust the plot type and style.
}
\examples{
library(STexampleData)

spe <- Visium_mouseCoronal()
colData(spe)$sum <- colSums(counts(spe))

# color by colData, highlight in-tissue spots
plotVisium(spe, annotate = "sum", highlight = "in_tissue")

# subset in-tissue spots
sub <- spe[, as.logical(colData(spe)$in_tissue)]

# color by feature counts, don't include image
rownames(sub) <- make.names(rowData(sub)$gene_name)
plotVisium(sub, annotate = "Gad2", assay = "counts")

}
\author{
Helena L. Crowell, with modifications by Lukas M. Weber and Yixing E.
  Dong
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotCoords.R
\name{plotCoords}
\alias{plotCoords}
\alias{plotSpots}
\title{plotCoords}
\usage{
plotCoords(
  spe,
  x_coord = NULL,
  y_coord = NULL,
  sample_id = NULL,
  in_tissue = "in_tissue",
  annotate = NULL,
  feature_names = NULL,
  assay_name = "counts",
  pal = NULL,
  point_shape = 16,
  point_size = 0.3,
  legend_position = "right",
  legend_point_size = 3,
  show_axes = FALSE,
  y_reverse = TRUE,
  text_by = NULL,
  text_by_size = 5,
  text_by_color = "black"
)

plotSpots(...)
}
\arguments{
\item{spe}{Input data, assumed to be a \code{SpatialExperiment} or
\code{SingleCellExperiment} object.}

\item{x_coord}{Name of column in \code{spatialCoords} (for a
\code{SpatialExperiment} input object) or \code{colData} (for a
\code{SingleCellExperiment} input object) containing x coordinates. Default
= NULL (for a \code{SpatialExperiment}, the first column of
\code{spatialCoords} will be selected in this case).}

\item{y_coord}{Name of column in \code{spatialCoords} (for a
\code{SpatialExperiment} input object) or \code{colData} (for a
\code{SingleCellExperiment} input object) containing y coordinates. Default
= NULL (for a \code{SpatialExperiment}, the second column of
\code{spatialCoords} will be selected in this case).}

\item{sample_id}{Name of column in \code{colData} containing sample IDs. This
argument is only required for datasets containing multiple samples (tissue
sections). If provided, samples will be shown in multiple panels using
facetting. Default = NULL.}

\item{in_tissue}{Name of column in \code{colData} identifying spots over
tissue (e.g. "in_tissue" for 10x Genomics Visium datasets). If this
argument is provided, only spots over tissue will be shown. Default =
"in_tissue". Set to NULL to display all spots.}

\item{annotate}{Variable to show as annotations. This may be discrete or
continuous. For a discrete variable (e.g. cluster labels), this should be
the name of a column in \code{colData} containing a character vector or
factor. For a continuous variable (e.g. a gene name), this should be an
entry in \code{feature_names}. Default = NULL.}

\item{feature_names}{Name of column in \code{rowData} containing names of
continuous features to plot (e.g. gene names). For example, set to
\code{feature_names = "gene_name"} if gene names are stored in a column
named \code{"gene_name"}. This argument is used if \code{annotate} is a
continuous variable. Default = NULL, in which case the row names of the
input object will be used.}

\item{assay_name}{Name of \code{assay} in input object containing values to
plot for a continuous variable. Default = "counts".}

\item{pal}{Color palette for annotations. Options for discrete values are
"libd_layer_colors", "Okabe-Ito", or any vector of color names or hex
values. For continuous values, provide a vector of length 2 for the low and
high range, e.g. c("gray90", "navy").}

\item{point_shape}{Point shape. Default = 16, which gives a circular shape
suitable for representing, for example, a Visium spot or Xenium cell. A
value of 15 gives a square shape suitable for Visium HD.}

\item{point_size}{Point size. Default = 0.3.}

\item{legend_position}{Legend position for discrete annotations. Options are
"left", "right", "top", "bottom", and "none". Default = "right".}

\item{legend_point_size}{Legend point size for discrete annotations. Default
= 3.}

\item{show_axes}{Whether to show axis titles, text, and ticks. Default =
FALSE.}

\item{y_reverse}{Whether to reverse y coordinates. This is usually required
for 10x Genomics Visium datasets when using the default coordinate values.
Default = TRUE. Set to FALSE if not needed, e.g. for other platforms.}

\item{text_by}{Column name of annotation labels to display over each cluster
of points. This will usually be the same as \code{annotate}. Alternatively,
another column may be used (e.g. with more readable classes or shorter
strings). Only used for discrete \code{annotate}. Default = NULL.}

\item{text_by_size}{Text size for annotation labels over each cluster.
Default = 5.}

\item{text_by_color}{Color name or hex code for annotation labels. Default =
"black".}

\item{...}{Not used.}
}
\value{
Returns a ggplot object, which may be further modified using ggplot
  functions.
}
\description{
Plotting functions for spatial transcriptomics data.
}
\details{
Function to create spot plot showing spatial locations in x-y coordinates
with optional annotations such as cluster labels, expression of a gene, or
quality control metrics.
}
\examples{
library(STexampleData)

# discrete annotations
spe <- Visium_humanDLPFC()
plotCoords(spe, annotate = "ground_truth")

# continuous annotations
spe <- Visium_mouseCoronal()
plotCoords(spe, annotate = "Gapdh", feature_names = "gene_name")

}
\author{
Lukas M. Weber and Yixing E. Dong
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotDimRed.R
\name{plotDimRed}
\alias{plotDimRed}
\title{plotDimRed}
\usage{
plotDimRed(
  spe,
  plot_type = c("UMAP", "PCA"),
  annotate = NULL,
  feature_names = NULL,
  assay_name = "counts",
  update_dimnames = TRUE,
  pal = NULL,
  point_size = 0.3,
  legend_point_size = 3,
  text_by = NULL,
  text_by_size = 5,
  text_by_color = "black"
)
}
\arguments{
\item{spe}{Input data, assumed to be a \code{SpatialExperiment} or
\code{SingleCellExperiment} object.}

\item{plot_type}{Type of reduced dimension plot. Possible options are "UMAP",
"PCA", or any other set of reduced dimensions stored in the input object.
Default = "UMAP".}

\item{annotate}{Variable to show as annotations. This may be discrete or
continuous. For a discrete variable (e.g. cluster labels), this should be
the name of a column in \code{colData} containing a character vector or
factor. For a continuous variable (e.g. a gene name), this should be an
entry in \code{feature_names}. Default = NULL.}

\item{feature_names}{Name of column in \code{rowData} containing names of
continuous features to plot (e.g. gene names). For example, set to
\code{feature_names = "gene_name"} if gene names are stored in a column
named \code{"gene_name"}. This argument is used if \code{annotate} is a
continuous variable. Default = NULL, in which case the row names of the
input object will be used.}

\item{assay_name}{Name of \code{assay} in input object containing values to
plot for a continuous variable. Default = "counts".}

\item{update_dimnames}{Whether to update column names of \code{reducedDims}
to default values for plotting. Default = TRUE.}

\item{pal}{Color palette for annotations. Options for discrete values are
"libd_layer_colors", "Okabe-Ito", or any vector of color names or hex
values. For continuous values, provide a vector of length 2 for the low and
high range, e.g. c("gray90", "navy").}

\item{point_size}{Point size. Default = 0.3.}

\item{legend_point_size}{Legend point size for discrete annotations. Default
= 3.}

\item{text_by}{Column name of annotation labels to display over each cluster
of points. This will usually be the same as \code{annotate}. Alternatively,
another column may be used (e.g. with more readable classes or shorter
strings). Only used for discrete \code{annotate}. Default = NULL.}

\item{text_by_size}{Text size for annotation labels over each cluster.
Default = 5.}

\item{text_by_color}{Color name or hex code for annotation labels. Default =
"black".}
}
\value{
Returns a ggplot object, which may be further modified using ggplot
  functions.
}
\description{
Plotting functions for spatial transcriptomics data.
}
\details{
Function to create reduced dimension plot (e.g. PCA or UMAP) with additional
optional annotations such as cluster labels, expression of a gene, or quality
control metrics.
}
\examples{
library(STexampleData)
spe <- Visium_humanDLPFC()

# select spots over tissue
spe <- spe[, colData(spe)$in_tissue == 1]

# use small subset of data for this example
n <- 200
set.seed(123)
spe <- spe[, sample(seq_len(ncol(spe)), n)]

# calculate logcounts
library(scran)
spe <- logNormCounts(spe)

# identify top highly variable genes (HVGs)
is_mito <- grepl("(^MT-)|(^mt-)", rowData(spe)$gene_name)
spe <- spe[!is_mito, ]
dec <- modelGeneVar(spe)
top_hvgs <- getTopHVGs(dec, prop = 0.1)

# run dimensionality reduction
library(scater)
set.seed(123)
spe <- runPCA(spe, subset_row = top_hvgs)
set.seed(123)
spe <- runUMAP(spe, dimred = "PCA")
colnames(reducedDim(spe, "UMAP")) <- paste0("UMAP", 1:2)

# generate plot
plotDimRed(spe, plot_type = "UMAP", annotate = "ground_truth")

}
\author{
Lukas M. Weber and Yixing E. Dong
}
